from typing import Any

import pytest

from checkov.common.util.data_structures_utils import find_in_dict


@pytest.mark.parametrize(
    "key_path,expected_value",
    [
        ("key_99", None),
        ("key_1/key_2/key_3", None),
        ("key_1/key_2/[10]/key_3", None),
        ("key_1/key_5", "string"),
        ("key_1/key_2/[0]/key_3", 1),
        ("key_1/key_2/[1]/key_4", True),
    ],
    ids=["key_not_exists", "nested_key_not_exists", "index_not_exists", "key", "index", "index_1"],
)
def test_find_in_dict(key_path: str, expected_value: Any) -> None:
    input_dict = {
        "key_1": {
            "key_2": [
                {
                    "key_3": 1,
                },
                {
                    "key_4": True,
                },
            ],
            "key_5": "string",
        }
    }

    # when
    actual_value = find_in_dict(input_dict, key_path)

    # then
    assert actual_value == expected_value



import os

from parameterized import parameterized_class

from tests.graph_utils.utils import PARAMETERIZED_GRAPH_FRAMEWORKS
from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class TestNotWithinSolver(TestBaseSolver):
    def setUp(self) -> None:
        self.checks_dir = TEST_DIRNAME
        super(TestNotWithinSolver, self).setUp()

    def test_basic_usage(self) -> None:
        root_folder = '../../../resources/public_virtual_machines'
        check_id = "NameNotWithin"
        should_pass = ['aws_subnet.subnet_not_public_ip']
        should_fail = ['aws_subnet.subnet_public_ip']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)



import unittest

from checkov.terraform.checks.resource.gcp.GKEDisableLegacyAuth import check
from checkov.common.models.enums import CheckResult


class GKEDisableLegacyAuth(unittest.TestCase):

    def test_failure(self):
        resource_conf = {'name': ['google_cluster'], 'enable_legacy_abac': [True]}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {'name': ['google_cluster']}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.azure.SynapseWorkspaceAdministratorLoginPasswordHidden import check
from checkov.terraform.runner import Runner

class TestSynapseWorkspaceAdministratorLoginPasswordHidden(unittest.TestCase):
    def test(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        test_files_dir = current_dir + "/example_SynapseWorkspaceAdministratorLoginPasswordHidden"
        report = runner.run(
            root_folder=test_files_dir, runner_filter=RunnerFilter(checks=[check.id])
        )
        summary = report.get_summary()

        passing_resources = {
            "azurerm_synapse_workspace.azurerm_synapse_workspace_pass",
        }
        failing_resources = {
            "azurerm_synapse_workspace.azurerm_synapse_workspace_fail",
        }

        passed_check_resources = set([c.resource for c in report.passed_checks])
        failed_check_resources = set([c.resource for c in report.failed_checks])

        self.assertEqual(summary["passed"], 1)
        self.assertEqual(summary["failed"], 1)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)


if __name__ == "__main__":
    unittest.main()



import os

from parameterized import parameterized_class

from tests.graph_utils.utils import PARAMETERIZED_GRAPH_FRAMEWORKS
from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class ConnectionSolver(TestBaseSolver):
    def setUp(self):
        self.checks_dir = TEST_DIRNAME
        super(ConnectionSolver, self).setUp()

    def test_and_connection(self):
        root_folder = '../../../resources/ec2_instance_network_interfaces'
        check_id = "AndComplexConnection"
        should_pass = ['aws_network_interface.network_interface_foo']
        should_fail = ['aws_network_interface.network_interface_goo', 'aws_instance.instance_bar', 'aws_instance.instance_foo']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)

    def test_multiple_connections(self):
        root_folder = '../../../resources/lb'
        check_id = "ALBConnectedToHTTPS"
        should_pass = []
        should_fail = ["aws_lb.lb_bad_1"]
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)



from __future__ import annotations

import inspect
from pathlib import Path

import pytest

from checkov.common.checks_infra.registry import get_graph_checks_registry
from checkov.common.bridgecrew.check_type import CheckType
from checkov.docs_generator import get_checks


def test_get_checks_returned_check_number():
    checks = get_checks(["all"])
    assert len(checks) > 0

    checks = get_checks()
    assert len(checks) > 0

    checks = get_checks(["example"])
    assert len(checks) == 0


@pytest.mark.parametrize(
    "input_frameworks,expected_frameworks",
    [
        (
            ["all"],
            {
                "Ansible",
                "Argo Workflows",
                "arm",
                "Azure Pipelines",
                "Bicep",
                "Cloudformation",
                "dockerfile",
                "Kubernetes",
                "secrets",
                "serverless",
                "Terraform",
                "github_configuration",
                "gitlab_configuration",
                "bitbucket_configuration",
                "bitbucket_pipelines",
                "circleci_pipelines",
                "github_actions",
                "OpenAPI",
                "gitlab_ci",
            },
        ),
        (
            None,
            {
                "Ansible",
                "Argo Workflows",
                "arm",
                "Azure Pipelines",
                "Bicep",
                "Cloudformation",
                "dockerfile",
                "Kubernetes",
                "secrets",
                "serverless",
                "Terraform",
                "github_configuration",
                "bitbucket_pipelines",
                "circleci_pipelines",
                "gitlab_configuration",
                "bitbucket_configuration",
                "github_actions",
                "OpenAPI",
                "gitlab_ci",
            },
        ),
        (["terraform"], {"Terraform"}),
        (["cloudformation", "serverless"], {"Cloudformation", "serverless"}),
    ],
    ids=["all", "none", "terraform", "multiple"],
)
def test_get_checks_returned_frameworks(input_frameworks: list[str] | None, expected_frameworks: set[str]):
    # when
    checks = get_checks(input_frameworks)

    # then
    actual_frameworks = {c[4] for c in checks}

    assert actual_frameworks == expected_frameworks


def test_get_checks_graph_registries():
    """
    For a runner that has graph_checks, the graph registry for that runner should be loaded and returned by the
    get_checks method.
    """
    checkov_runners = [value for attr, value in CheckType.__dict__.items() if not attr.startswith("__")]
    for runner in checkov_runners:
        graph_registry = get_graph_checks_registry(runner)
        if Path(graph_registry.checks_dir).is_dir():
            assert f'get_graph_checks_registry("{runner}")' in inspect.getsource(get_checks)



import unittest
from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.aws.IAMStarResourcePolicyDocument import check
from checkov.terraform.checks.utils.base_cloudsplaining_iam_scanner import BaseTerraformCloudsplainingIAMScanner
from checkov.terraform.runner import Runner


class TestIAMStarResourcePolicyDocument(unittest.TestCase):
    def setUp(self) -> None:
        # make sure nothing is in the cache
        BaseTerraformCloudsplainingIAMScanner.policy_document_cache = {}

    def tearDown(self) -> None:
        BaseTerraformCloudsplainingIAMScanner.policy_document_cache = {}

    def test(self):
        # given
        test_files_dir = Path(__file__).parent / "example_IAMStarResourcePolicyDocument"

        # when
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))

        # then
        summary = report.get_summary()

        passing_resources = {
            "aws_iam_policy.pass",
            "aws_iam_policy.pass_unrestrictable",
            "aws_iam_role_policy.pass",
            "aws_iam_user_policy.pass",
            "aws_iam_group_policy.pass",
            "aws_ssoadmin_permission_set_inline_policy.pass"
        }
        failing_resources = {
            "aws_iam_policy.fail",
            "aws_iam_role_policy.fail",
            "aws_iam_user_policy.fail",
            "aws_iam_group_policy.fail",
            "aws_ssoadmin_permission_set_inline_policy.fail"
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary["passed"], len(passing_resources))
        self.assertEqual(summary["failed"], len(failing_resources))
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)


if __name__ == "__main__":
    unittest.main()




from __future__ import annotations

import os
from unittest import mock

import pytest
import responses

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.secrets.consts import VerifySecretsResult


@mock.patch.dict(os.environ, {"CKV_VALIDATE_SECRETS": "true"})
def test_verify_secrets_insufficient_params_skip_download() -> None:
    from checkov.common.bridgecrew.platform_integration import bc_integration
    bc_integration.skip_download = True
    bc_integration.bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"

    from checkov.secrets.runner import Runner
    from checkov.common.output.report import Report
    result = Runner().verify_secrets(Report(check_type=CheckType.SECRETS), "")

    assert result == VerifySecretsResult.INSUFFICIENT_PARAMS


@mock.patch.dict(os.environ, {"CKV_VALIDATE_SECRETS": "true"})
def test_verify_secrets_insufficient_params_no_api_key() -> None:
    from checkov.common.bridgecrew.platform_integration import bc_integration
    bc_integration.bc_api_key = None

    from checkov.secrets.runner import Runner
    from checkov.common.output.report import Report
    result = Runner().verify_secrets(Report(check_type=CheckType.SECRETS), "")

    assert result == VerifySecretsResult.INSUFFICIENT_PARAMS


def test_verify_secrets_insufficient_params_no_flag() -> None:
    # Not setting CKV_VALIDATE_SECRETS env var
    from checkov.common.bridgecrew.platform_integration import bc_integration
    bc_integration.bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"

    from checkov.secrets.runner import Runner
    from checkov.common.output.report import Report
    result = Runner().verify_secrets(Report(check_type=CheckType.SECRETS), "")

    assert result == VerifySecretsResult.INSUFFICIENT_PARAMS

@mock.patch.dict(os.environ, {"CKV_VALIDATE_SECRETS": "true"})
def test_verify_secrets_insufficient_params_tenant_config_overrides_true_flag() -> None:
    from checkov.common.bridgecrew.platform_integration import bc_integration
    bc_integration.bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"
    bc_integration.customer_run_config_response = {'tenantConfig': {'secretsValidate': False}}

    from checkov.secrets.runner import Runner
    from checkov.common.output.report import Report
    result = Runner().verify_secrets(Report(check_type=CheckType.SECRETS), "")

    assert result == VerifySecretsResult.INSUFFICIENT_PARAMS


@mock.patch.dict(os.environ, {"CKV_VALIDATE_SECRETS": "false"})
def test_verify_secrets_insufficient_params_tenant_config_overrides_false_flag() -> None:
    from checkov.common.bridgecrew.platform_integration import bc_integration
    bc_integration.bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"
    bc_integration.customer_run_config_response = {'tenantConfig': {'secretsValidate': True}}
    bc_integration.skip_download = False

    from checkov.secrets.runner import Runner
    from checkov.common.output.report import Report
    result = Runner().verify_secrets(Report(check_type=CheckType.SECRETS), "")

    assert result != VerifySecretsResult.INSUFFICIENT_PARAMS

@mock.patch.dict(os.environ, {"CKV_VALIDATE_SECRETS": "false"})
def test_verify_secrets_insufficient_params_tenant_config_missing_false_flag() -> None:
    from checkov.common.bridgecrew.platform_integration import bc_integration
    bc_integration.bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"
    bc_integration.customer_run_config_response = {'tenantConfig': {'mock': True}}
    bc_integration.skip_download = False

    from checkov.secrets.runner import Runner
    from checkov.common.output.report import Report
    result = Runner().verify_secrets(Report(check_type=CheckType.SECRETS), "")

    assert result == VerifySecretsResult.INSUFFICIENT_PARAMS

@mock.patch.dict(os.environ, {"CKV_VALIDATE_SECRETS": "true"})
def test_verify_secrets_insufficient_params_tenant_config_missing_true_flag() -> None:
    from checkov.common.bridgecrew.platform_integration import bc_integration
    bc_integration.bc_api_key = "abcd1234-abcd-1234-abcd-1234abcd1234"
    bc_integration.customer_run_config_response = {'tenantConfig': {'mock': True}}
    bc_integration.skip_download = False

    from checkov.secrets.runner import Runner
    from checkov.common.output.report import Report
    result = Runner().verify_secrets(Report(check_type=CheckType.SECRETS), "")

    assert result != VerifySecretsResult.INSUFFICIENT_PARAMS

@responses.activate
@mock.patch.dict(os.environ, {"CKV_VALIDATE_SECRETS": "true"})
@pytest.mark.parametrize(
    "status_code",
    [
        (500,),
        (400,),
    ]
)
def test_verify_secrets_failure(mock_bc_integration, status_code: int) -> None:
    responses.add(
        method=responses.POST,
        url=f"{mock_bc_integration.api_url}/api/v1/secrets/reportVerification",
        json={},
        status=status_code
    )

    from checkov.secrets.runner import Runner
    from checkov.common.output.report import Report
    result = Runner().verify_secrets(Report(check_type=CheckType.SECRETS), "")

    assert result == VerifySecretsResult.FAILURE


@responses.activate
@mock.patch.dict(os.environ, {"CKV_VALIDATE_SECRETS": "true"})
def test_verify_secrets(mock_bc_integration, secrets_report) -> None:
    violation_id_to_verify_status = {"VIOLATION_1": "Privileged",
                                     "VIOLATION_2": "Valid",
                                     "VIOLATION_3": "Invalid",
                                     "VIOLATION_4": "Unknown"}
    verified_report = [
            {
                "violationId": "VIOLATION_1",
                "resourceId": "mock:RESOURCE_1",
                "status": "Privileged"
            },
            {
                "violationId": "VIOLATION_2",
                "resourceId": "mock:RESOURCE_2",
                "status": "Valid"
            },
            {
                "violationId": "VIOLATION_3",
                "resourceId": "mock:RESOURCE_3",
                "status": "Invalid"
            },
            {
                "violationId": "VIOLATION_4",
                "resourceId": "mock:RESOURCE_4",
                "status": "Unknown"
            }
        ]

    responses.add(
        method=responses.POST,
        url=f"{mock_bc_integration.api_url}/api/v1/secrets/reportVerification",
        json={'verificationReportSignedUrl': 'mock'},
        status=200
    )

    from checkov.secrets.runner import Runner
    runner = Runner()
    runner.get_json_verification_report = lambda x: verified_report
    result = runner.verify_secrets(secrets_report, "path/to/enriched/secrets")

    assert result == VerifySecretsResult.SUCCESS
    for check in secrets_report.failed_checks:
        if hasattr(check, "validation_status"):
            assert check.validation_status == violation_id_to_verify_status[check.bc_check_id]
        else:
            raise Exception("Secrets record should have a validation status attribute")

    for check in secrets_report.passed_checks:
        if hasattr(check, "validation_status"):
            assert check.validation_status == 'mock'


@responses.activate
@mock.patch.dict(os.environ, {"CKV_VALIDATE_SECRETS": "true"})
def test_runner_verify_secrets(mock_bc_integration, mock_metadata_integration):
    current_dir = os.path.dirname(os.path.realpath(__file__))
    valid_dir_path = current_dir + "/resources/cfn"

    rel_resource_path = '/secret.yml'
    resource_id = '25910f981e85ca04baf359199dd0bd4a3ae738b6'
    verified_report = [
        {
            "violationId": "BC_GIT_2",
            "resourceId": f"{rel_resource_path}:{resource_id}",
            "status": "Valid"
        }
    ]

    responses.add(
        method=responses.POST,
        url=f"{mock_bc_integration.api_url}/api/v1/secrets/reportVerification",
        json={'verificationReportSignedUrl': 'mock'},
        status=200
    )

    from checkov.secrets.runner import Runner
    runner = Runner()
    mock_bc_integration.persist_enriched_secrets = lambda x: 'mock'
    mock_bc_integration.bc_api_key = 'mock'
    runner.get_json_verification_report = lambda x: verified_report

    from checkov.runner_filter import RunnerFilter
    report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                        runner_filter=RunnerFilter(framework=['secrets']))

    for check in report.failed_checks:
        if check.file_path == rel_resource_path and check.resource == resource_id:
            assert check.validation_status == 'Valid'
        else:
            assert check.validation_status == 'Unavailable'



from aws_cdk import core
from aws_cdk import aws_redshift as redshift

class RedshiftClusterStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create an Amazon Redshift cluster
        redshift_cluster = redshift.CfnCluster(
            self,
            "MyRedshiftCluster",
            cluster_identifier="my-redshift-cluster",
            master_username="admin",
            master_user_password="MySecurePassword123",  # checkov:skip=CKV_SECRET_6 test secret
            node_type="dc2.large",
            cluster_type="single-node",
        )

app = core.App()
RedshiftClusterStack(app, "RedshiftClusterStack")
app.synth()



import unittest

import hcl2

from checkov.terraform.checks.resource.gcp.GoogleRoleServiceAccountUser import check
from checkov.common.models.enums import CheckResult


class TestGoogleRoleServiceAccountUser(unittest.TestCase):

    def test_failure_1(self):
        hcl_res = hcl2.loads("""
            resource "google_project_iam_member" "project" {
              project = "your-project-id"
              role    = "roles/iam.serviceAccountUser"
              member  = "user:jane@example.com"
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_project_iam_member']['project']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_2(self):
        hcl_res = hcl2.loads("""
            resource "google_project_iam_binding" "project" {
              project = "your-project-id"
              role    = "roles/iam.serviceAccountTokenCreator"

              members = [
                "user:jane@example.com",
              ]
            }
                """)
        resource_conf = hcl_res['resource'][0]['google_project_iam_binding']['project']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
        resource "google_project_iam_member" "project" {
          project = "your-project-id"
          role    = "roles/editor"
          member  = "user:jane@example.com"
        }
                        """)
        resource_conf = hcl_res['resource'][0]['google_project_iam_member']['project']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.azure.VMScaleSetsAutoOSImagePatchingEnabled import check


class TestVMScaleSetsAutoOSImagePatchingEnabled(unittest.TestCase):

    def test_failure1(self):
        hcl_res = hcl2.loads("""
        resource "azurerm_virtual_machine_scale_set" "example" {
          name                = "mytestscaleset-1"
          location            = azurerm_resource_group.example.location
          resource_group_name = azurerm_resource_group.example.name
        
          # automatic rolling upgrade
          upgrade_policy_mode  = "Rolling"
        
          rolling_upgrade_policy {
            max_batch_instance_percent              = 20
            max_unhealthy_instance_percent          = 20
            max_unhealthy_upgraded_instance_percent = 5
            pause_time_between_batches              = "PT0S"
          }
        
          # required when using rolling upgrade policy
          health_probe_id = azurerm_lb_probe.example.id
        
          sku {
            name     = "Standard_F2"
            tier     = "Standard"
            capacity = 2
          }
        
          storage_profile_image_reference {
            publisher = "Canonical"
            offer     = "UbuntuServer"
            sku       = "16.04-LTS"
            version   = "latest"
          }
        
          storage_profile_os_disk {
            name              = ""
            caching           = "ReadWrite"
            create_option     = "FromImage"
            managed_disk_type = "Standard_LRS"
          }
        
          storage_profile_data_disk {
            lun           = 0
            caching       = "ReadWrite"
            create_option = "Empty"
            disk_size_gb  = 10
          }
        
          os_profile {
            computer_name_prefix = "testvm"
            admin_username       = "myadmin"
          }
        
          os_profile_linux_config {
            disable_password_authentication = true
        
            ssh_keys {
              path     = "/home/myadmin/.ssh/authorized_keys"
              key_data = file("~/.ssh/demo_key.pub")
            }
          }
        
          network_profile {
            name    = "terraformnetworkprofile"
            primary = true
        
            ip_configuration {
              name                                   = "TestIPConfiguration"
              primary                                = true
              subnet_id                              = azurerm_subnet.example.id
              load_balancer_backend_address_pool_ids = [azurerm_lb_backend_address_pool.bpepool.id]
              load_balancer_inbound_nat_rules_ids    = [azurerm_lb_nat_pool.lbnatpool.id]
            }
          }
        
          tags = {
            environment = "staging"
          }
        }

                """)
        resource_conf = hcl_res['resource'][0]['azurerm_virtual_machine_scale_set']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure2(self):
        hcl_res = hcl2.loads("""
        resource "azurerm_virtual_machine_scale_set" "example" {
          name                = "mytestscaleset-1"
          location            = azurerm_resource_group.example.location
          resource_group_name = azurerm_resource_group.example.name

          # automatic rolling upgrade
          automatic_os_upgrade = false
          upgrade_policy_mode  = "Rolling"

          rolling_upgrade_policy {
            max_batch_instance_percent              = 20
            max_unhealthy_instance_percent          = 20
            max_unhealthy_upgraded_instance_percent = 5
            pause_time_between_batches              = "PT0S"
          }

          # required when using rolling upgrade policy
          health_probe_id = azurerm_lb_probe.example.id

          sku {
            name     = "Standard_F2"
            tier     = "Standard"
            capacity = 2
          }

          storage_profile_image_reference {
            publisher = "Canonical"
            offer     = "UbuntuServer"
            sku       = "16.04-LTS"
            version   = "latest"
          }

          storage_profile_os_disk {
            name              = ""
            caching           = "ReadWrite"
            create_option     = "FromImage"
            managed_disk_type = "Standard_LRS"
          }

          storage_profile_data_disk {
            lun           = 0
            caching       = "ReadWrite"
            create_option = "Empty"
            disk_size_gb  = 10
          }

          os_profile {
            computer_name_prefix = "testvm"
            admin_username       = "myadmin"
          }

          os_profile_linux_config {
            disable_password_authentication = true

            ssh_keys {
              path     = "/home/myadmin/.ssh/authorized_keys"
              key_data = file("~/.ssh/demo_key.pub")
            }
          }

          network_profile {
            name    = "terraformnetworkprofile"
            primary = true

            ip_configuration {
              name                                   = "TestIPConfiguration"
              primary                                = true
              subnet_id                              = azurerm_subnet.example.id
              load_balancer_backend_address_pool_ids = [azurerm_lb_backend_address_pool.bpepool.id]
              load_balancer_inbound_nat_rules_ids    = [azurerm_lb_nat_pool.lbnatpool.id]
            }
          }

          tags = {
            environment = "staging"
          }
        }

                """)
        resource_conf = hcl_res['resource'][0]['azurerm_virtual_machine_scale_set']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure3(self):
        hcl_res = hcl2.loads("""
        resource "azurerm_virtual_machine_scale_set" "example" {
          name                = "mytestscaleset-1"
          location            = azurerm_resource_group.example.location
          resource_group_name = azurerm_resource_group.example.name

          # automatic rolling upgrade
          automatic_os_upgrade = true
          upgrade_policy_mode  = "Rolling"

          rolling_upgrade_policy {
            max_batch_instance_percent              = 20
            max_unhealthy_instance_percent          = 20
            max_unhealthy_upgraded_instance_percent = 5
            pause_time_between_batches              = "PT0S"
          }

          # required when using rolling upgrade policy
          health_probe_id = azurerm_lb_probe.example.id

          sku {
            name     = "Standard_F2"
            tier     = "Standard"
            capacity = 2
          }

          storage_profile_image_reference {
            publisher = "Canonical"
            offer     = "UbuntuServer"
            sku       = "16.04-LTS"
            version   = "latest"
          }

          storage_profile_os_disk {
            name              = ""
            caching           = "ReadWrite"
            create_option     = "FromImage"
            managed_disk_type = "Standard_LRS"
          }

          storage_profile_data_disk {
            lun           = 0
            caching       = "ReadWrite"
            create_option = "Empty"
            disk_size_gb  = 10
          }

          os_profile {
            computer_name_prefix = "testvm"
            admin_username       = "myadmin"
          }

          os_profile_linux_config {
            disable_password_authentication = true

            ssh_keys {
              path     = "/home/myadmin/.ssh/authorized_keys"
              key_data = file("~/.ssh/demo_key.pub")
            }
          }

          network_profile {
            name    = "terraformnetworkprofile"
            primary = true

            ip_configuration {
              name                                   = "TestIPConfiguration"
              primary                                = true
              subnet_id                              = azurerm_subnet.example.id
              load_balancer_backend_address_pool_ids = [azurerm_lb_backend_address_pool.bpepool.id]
              load_balancer_inbound_nat_rules_ids    = [azurerm_lb_nat_pool.lbnatpool.id]
            }
          }

          tags = {
            environment = "staging"
          }
        }

                """)
        resource_conf = hcl_res['resource'][0]['azurerm_virtual_machine_scale_set']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure4(self):
        hcl_res = hcl2.loads("""
        resource "azurerm_virtual_machine_scale_set" "example" {
          name                = "mytestscaleset-1"
          location            = azurerm_resource_group.example.location
          resource_group_name = azurerm_resource_group.example.name

          # automatic rolling upgrade
          automatic_os_upgrade = true
          upgrade_policy_mode  = "Rolling"
          
          os_profile_windows_config {
            enable_automatic_upgrades = false
          }

          rolling_upgrade_policy {
            max_batch_instance_percent              = 20
            max_unhealthy_instance_percent          = 20
            max_unhealthy_upgraded_instance_percent = 5
            pause_time_between_batches              = "PT0S"
          }

          # required when using rolling upgrade policy
          health_probe_id = azurerm_lb_probe.example.id

          sku {
            name     = "Standard_F2"
            tier     = "Standard"
            capacity = 2
          }

          storage_profile_image_reference {
            publisher = "Canonical"
            offer     = "UbuntuServer"
            sku       = "16.04-LTS"
            version   = "latest"
          }

          storage_profile_os_disk {
            name              = ""
            caching           = "ReadWrite"
            create_option     = "FromImage"
            managed_disk_type = "Standard_LRS"
          }

          storage_profile_data_disk {
            lun           = 0
            caching       = "ReadWrite"
            create_option = "Empty"
            disk_size_gb  = 10
          }

          os_profile {
            computer_name_prefix = "testvm"
            admin_username       = "myadmin"
          }

          os_profile_linux_config {
            disable_password_authentication = true

            ssh_keys {
              path     = "/home/myadmin/.ssh/authorized_keys"
              key_data = file("~/.ssh/demo_key.pub")
            }
          }

          network_profile {
            name    = "terraformnetworkprofile"
            primary = true

            ip_configuration {
              name                                   = "TestIPConfiguration"
              primary                                = true
              subnet_id                              = azurerm_subnet.example.id
              load_balancer_backend_address_pool_ids = [azurerm_lb_backend_address_pool.bpepool.id]
              load_balancer_inbound_nat_rules_ids    = [azurerm_lb_nat_pool.lbnatpool.id]
            }
          }

          tags = {
            environment = "staging"
          }
        }

                """)
        resource_conf = hcl_res['resource'][0]['azurerm_virtual_machine_scale_set']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure5(self):
        hcl_res = hcl2.loads("""
        resource "azurerm_virtual_machine_scale_set" "example" {
          name                = "mytestscaleset-1"
          location            = azurerm_resource_group.example.location
          resource_group_name = azurerm_resource_group.example.name

          # automatic rolling upgrade
          upgrade_policy_mode  = "Rolling"

          os_profile_windows_config {
            enable_automatic_upgrades = false
          }

          rolling_upgrade_policy {
            max_batch_instance_percent              = 20
            max_unhealthy_instance_percent          = 20
            max_unhealthy_upgraded_instance_percent = 5
            pause_time_between_batches              = "PT0S"
          }

          # required when using rolling upgrade policy
          health_probe_id = azurerm_lb_probe.example.id

          sku {
            name     = "Standard_F2"
            tier     = "Standard"
            capacity = 2
          }

          storage_profile_image_reference {
            publisher = "Canonical"
            offer     = "UbuntuServer"
            sku       = "16.04-LTS"
            version   = "latest"
          }

          storage_profile_os_disk {
            name              = ""
            caching           = "ReadWrite"
            create_option     = "FromImage"
            managed_disk_type = "Standard_LRS"
          }

          storage_profile_data_disk {
            lun           = 0
            caching       = "ReadWrite"
            create_option = "Empty"
            disk_size_gb  = 10
          }

          os_profile {
            computer_name_prefix = "testvm"
            admin_username       = "myadmin"
          }

          os_profile_linux_config {
            disable_password_authentication = true

            ssh_keys {
              path     = "/home/myadmin/.ssh/authorized_keys"
              key_data = file("~/.ssh/demo_key.pub")
            }
          }

          network_profile {
            name    = "terraformnetworkprofile"
            primary = true

            ip_configuration {
              name                                   = "TestIPConfiguration"
              primary                                = true
              subnet_id                              = azurerm_subnet.example.id
              load_balancer_backend_address_pool_ids = [azurerm_lb_backend_address_pool.bpepool.id]
              load_balancer_inbound_nat_rules_ids    = [azurerm_lb_nat_pool.lbnatpool.id]
            }
          }

          tags = {
            environment = "staging"
          }
        }

                """)
        resource_conf = hcl_res['resource'][0]['azurerm_virtual_machine_scale_set']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_virtual_machine_scale_set" "example" {
          name                = "mytestscaleset-1"
          location            = azurerm_resource_group.example.location
          resource_group_name = azurerm_resource_group.example.name

          # automatic rolling upgrade
          automatic_os_upgrade = true
          upgrade_policy_mode  = "Rolling"
          
          os_profile_windows_config {
            enable_automatic_upgrades = true
          }

          rolling_upgrade_policy {
            max_batch_instance_percent              = 20
            max_unhealthy_instance_percent          = 20
            max_unhealthy_upgraded_instance_percent = 5
            pause_time_between_batches              = "PT0S"
          }

          # required when using rolling upgrade policy
          health_probe_id = azurerm_lb_probe.example.id

          sku {
            name     = "Standard_F2"
            tier     = "Standard"
            capacity = 2
          }

          storage_profile_image_reference {
            publisher = "Canonical"
            offer     = "UbuntuServer"
            sku       = "16.04-LTS"
            version   = "latest"
          }

          storage_profile_os_disk {
            name              = ""
            caching           = "ReadWrite"
            create_option     = "FromImage"
            managed_disk_type = "Standard_LRS"
          }

          storage_profile_data_disk {
            lun           = 0
            caching       = "ReadWrite"
            create_option = "Empty"
            disk_size_gb  = 10
          }

          os_profile {
            computer_name_prefix = "testvm"
            admin_username       = "myadmin"
          }

          os_profile_linux_config {
            disable_password_authentication = true

            ssh_keys {
              path     = "/home/myadmin/.ssh/authorized_keys"
              key_data = file("~/.ssh/demo_key.pub")
            }
          }

          network_profile {
            name    = "terraformnetworkprofile"
            primary = true

            ip_configuration {
              name                                   = "TestIPConfiguration"
              primary                                = true
              subnet_id                              = azurerm_subnet.example.id
              load_balancer_backend_address_pool_ids = [azurerm_lb_backend_address_pool.bpepool.id]
              load_balancer_inbound_nat_rules_ids    = [azurerm_lb_nat_pool.lbnatpool.id]
            }
          }

          tags = {
            environment = "staging"
          }
        }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_virtual_machine_scale_set']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest
import hcl2

from checkov.terraform.checks.resource.aws.RDSPubliclyAccessible import check
from checkov.common.models.enums import CheckResult


class TestRDSPubliclyAccessible(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
resource aws_rds_cluster_instance "rds_cluster_public" {
  cluster_identifier = "id"
  instance_class = "foo-bar"
  publicly_accessible = true
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_rds_cluster_instance']['rds_cluster_public']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
        resource aws_rds_cluster_instance "rds_cluster_public" {
          cluster_identifier = "id"
          instance_class = "foo-bar"
          publicly_accessible = false
        }
        """)
        resource_conf = hcl_res['resource'][0]['aws_rds_cluster_instance']['rds_cluster_public']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success2(self):
        hcl_res = hcl2.loads("""
           resource aws_rds_cluster_instance "rds_cluster_public" {
             cluster_identifier = "id"
             instance_class = "foo-bar"
           }
           """)
        resource_conf = hcl_res['resource'][0]['aws_rds_cluster_instance']['rds_cluster_public']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import json
import unittest

import hcl2


# This group of tests is used to confirm assumptions about how the hcl2 library parses into json.
# We want to make sure important assumptions are caught if behavior changes.
class TestHCL2LoadAssumptions(unittest.TestCase):
    def test_ternary(self):
        # Ternary and removal of parens are interesting things here
        tf = '''
        resource "aws_instance" "foo" {
          metadata_options {
            http_tokens = (var.metadata_http_tokens_required) ? "required" : "optional"
          }
        }'''
        expect = {
            "resource": [{
                "aws_instance": {
                    "foo": {
                        "metadata_options": [{
                            "http_tokens": ['${var.metadata_http_tokens_required ? "required" : "optional"}']
                        }],
                        "__start_line__": 2,
                        "__end_line__": 6,
                    }
                }
            }]
        }
        self.go(tf, expect)

    def test_tfvars(self):
        tf = '''
        VERSIONING = true
        CHECKOV = "awesome"
        '''
        expect = {
            "VERSIONING": [True],
            "CHECKOV": ["awesome"]
        }
        self.go(tf, expect)

    def test_multiline_function(self):
        tf = '''
        locals {
           a_string = merge(
             local.foo,
             {a="b"}
           )
        }'''
        expect = {
            "locals": [
                {
                    "a_string": ["${merge(local.foo,{'a': 'b'})}"],
                    "__start_line__": 2,
                    "__end_line__": 7,
                }
            ]
        }
        self.go(tf, expect)

    def test_string_with_quotes(self):
        tf = '''
        locals {
           a_string = "Quotes are \\"fun\\"!"
        }'''
        expect = {
            "locals": [
                {
                    "a_string": ["Quotes are \\\"fun\\\"!"],
                    #                        __--
                    #                        |  |
                    #                backslash  quote
                    "__start_line__": 2,
                    "__end_line__": 4,
                }
            ]
        }
        self.go(tf, expect)

    def test_inner_quoting(self):
        tf = '''
        locals {
          evil_strings1 = merge({a="}, evil"})
        }'''
        expect = {
            "locals": [
                {
                    "evil_strings1": ["${merge({'a': '}, evil'})}"],
                    "__start_line__": 2,
                    "__end_line__": 4,
                },
            ]
        }
        self.go(tf, expect)

    def test_merge_with_inner_var(self):
        tf = '''
        resource "aws_s3_bucket" "foo" {
          tags = merge(local.common_tags, local.common_data_tags, {Name = "my-thing-${var.ENVIRONMENT}-${var.REGION}"})
        }'''
        expect = {
            "resource": [
                {
                    "aws_s3_bucket": {
                        "foo": {
                            "tags": ["${merge(local.common_tags,local.common_data_tags,{'Name': 'my-thing-${var.ENVIRONMENT}-${var.REGION}'})}"],
                            "__start_line__": 2,
                            "__end_line__": 4,
                        }
                    }
                }
            ]
        }
        self.go(tf, expect)

    def test_variable_block(self):
        tf = '''
        variable "my_var" {
          type = string
          default = "my_default_value"
        }'''
        expect = {
            "variable": [
                {
                    "my_var": {
                        "type": ["${string}"],  # NOTE: wrapped in eval markers
                        "default": ["my_default_value"],
                        "__start_line__": 2,
                        "__end_line__": 5,
                    }
                }
            ]
        }
        self.go(tf, expect)

    def test_module_block(self):
        tf = '''
        module "bucket" {
          source   = "./bucket"
          name     = "module_bucket"
          BLAH     = "a value"
        }'''
        expect = {
            "module": [
                {
                    "bucket": {
                        "source": ["./bucket"],
                        "name": ["module_bucket"],
                        "BLAH": ["a value"],
                        "__start_line__": 2,
                        "__end_line__": 6,
                    }
                }
            ]
        }
        self.go(tf, expect)

    def test_raw_assignment(self):
        tf = 'my_var = "my_value"\n'
        expect = {
            "my_var": ["my_value"]
        }
        self.go(tf, expect)

    def test_raw_assignment_true_string(self):
        tf = 'my_var = "true"\n'
        expect = {
            "my_var": ["true"]
        }
        self.go(tf, expect)

    def test_raw_assignment_false_string(self):
        tf = 'my_var = "false"\n'
        expect = {
            "my_var": ["false"]
        }
        self.go(tf, expect)

    def test_raw_assignment_1_string(self):
        tf = 'my_var = "1"\n'
        expect = {
            "my_var": ["1"]
        }
        self.go(tf, expect)

    def test_raw_assignment_0_string(self):
        tf = 'my_var = "0"\n'
        expect = {
            "my_var": ["0"]
        }
        self.go(tf, expect)

    def test_map_separators(self):
        tf = '''
        locals {
          INTS = tomap({"a" = 1, "b" = 2})
        }'''
        expect = {
            "locals": [
                {
                    "INTS": ["${tomap({'a': 1, 'b': 2})}"],  # WHA?? Equals to colons? Okay...
                    "__start_line__": 2,
                    "__end_line__": 4,
                }
            ]
        }
        self.go(tf, expect)

    # from the "maze_of_variables" scenario
    def test_maze_of_variables(self):
        tf = '''
        variable "gratuitous_var_default" {
          type = string
          default = "-yay"
        }

        variable "input" {
          default = "module-input"
        }
         
        locals {
          NAME = {
            "module-input-bucket" = "mapped-bucket-name"
          }
          TAIL = "works"
        }
          
        module "bucket" {
          source   = "./bucket"
          name     = var.input
        }
        
        resource "aws_s3_bucket" "example2" {
          #             resolves to: mapped-bucket-name
          #             |            resolves to: module-input-bucket
          #             |            |                              resolves to: works
          #             |            |                              |           resolves to: -yay
          #             |            |                              |           |
          #             v            v                              v           v
          bucket = "${local.NAME[${module.bucket.bucket_name}]}-${local.TAIL}${var.gratuitous_var_default}"
          # final result: mapped-bucket-name-works-yay
        }'''
        expect = {
            "variable": [
                {
                    "gratuitous_var_default": {
                        "type": ["${string}"],              # NOTE: wrapped in eval markers
                        "default": ["-yay"],
                        "__start_line__": 2,
                        "__end_line__": 5,
                    }
                },
                {
                    "input": {
                        "default": ["module-input"],
                        "__start_line__": 7,
                        "__end_line__": 9,
                    }
                }
            ],
            "locals": [
                {
                    "NAME": [{
                        "module-input-bucket": "mapped-bucket-name"
                    }],
                    "TAIL": ["works"],
                    "__start_line__": 11,
                    "__end_line__": 16,
                }
            ],
            "module": [
                {
                    "bucket": {
                        "source": ["./bucket"],
                        "name": ["${var.input}"],  # NOTE: wrapped in eval markers
                        "__start_line__": 18,
                        "__end_line__": 21,
                    }
                }
            ],
            "resource": [
                {
                    "aws_s3_bucket": {
                        "example2": {
                            "bucket": ["${local.NAME[${module.bucket.bucket_name}]}-${local.TAIL}${var.gratuitous_var_default}"],
                            "__start_line__": 23,
                            "__end_line__": 32,
                        }
                    }
                }
            ]
        }
        self.go(tf, expect)

    @staticmethod
    def go(tf, expected_result):
        actual_result = hcl2.loads(tf)
        assert actual_result == expected_result, "Results mismatch:\n" \
                                                 "** EXPECTED **\n" \
                                                 f"{json.dumps(expected_result, indent=2)}\n" \
                                                 f"** ACTUAL **\n" \
                                                 f"{json.dumps(actual_result, indent=2)}"

    def test_math(self):
        tf = "four = 2 + 2"
        expect = {
            "four": ["${2 + 2}"]
        }
        self.go(tf, expect)

    def test_weird_ternary_string_clipping(self):
        tf = 'bool_string_false = "false" ? "wrong" : "correct"'
        expect = {
            "bool_string_false": ['${"false" ? "wrong" : "correct"}']
        }
        self.go(tf, expect)

    def test_splat_expression(self):
        tf = 'instances = flatten(aws_instance.ubuntu[*].id)'
        expect = {
            'instances': ["${flatten(aws_instance.ubuntu[*].id)}"]
        }
        self.go(tf, expect)

    def test_provider_function(self):
        tf = "name2 = provider::test2::test(\"a\")"
        expect = {
            "name2": ["${provider::test2::test(\"a\")}"],
        }
        self.go(tf, expect)



import unittest

from checkov.terraform.checks.resource.gcp.GoogleCloudDNSKeySpecsRSASHA1 import check
from checkov.common.models.enums import CheckResult


class TestCloudDNSKeySpecsRSASHA1(unittest.TestCase):

    def test_failure_zone_signing(self):
        resource_conf = {"name": ["example-zone"],
                         "dns_name": ["example-de13he3.com."],
                         "description": ["Example DNS zone"],
                         "dnssec_config": [{
                             "state": ["on"],
                             "default_key_specs": [
                                 {"algorithm": ["rsasha1"], "key_type": ["zoneSigning"], "key_length": "1024"},
                                 {"algorithm": ["rsasha256"], "key_type": ["keySigning"], "key_length": "2048"},
                             ]
                         }]
                         }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_key_signing(self):
        resource_conf = {"name": ["example-zone"],
                         "dns_name": ["example-de13he3.com."],
                         "description": ["Example DNS zone"],
                         "dnssec_config": [{
                             "state": ["on"],
                             "default_key_specs": [
                                 {"algorithm": ["rsasha256"], "key_type": ["zoneSigning"], "key_length": "1024"},
                                 {"algorithm": ["rsasha1"], "key_type": ["keySigning"], "key_length": "2048"},
                             ]
                         }]
                         }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {"name": ["example-zone"],
                         "dns_name": ["example-de13he3.com."],
                         "description": ["Example DNS zone"],
                         "dnssec_config": [{
                             "state": ["on"],
                             "default_key_specs": [
                                 {"algorithm": ["rsasha256"], "key_type": ["zoneSigning"], "key_length": "1024"},
                                 {"algorithm": ["rsasha256"], "key_type": ["keySigning"], "key_length": "2048"},
                             ]
                         }]
                         }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_default_config(self):
        resource_conf = {"name": ["example-zone"],
                         "dns_name": ["example-de13he3.com."],
                         "description": ["Example DNS zone"],
                         "dnssec_config": [{"state": ["on"]}]
                         }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from __future__ import annotations

import io
import logging
import multiprocessing
import os
import pathlib
import platform
import re
import shutil
import subprocess  # nosec
import tempfile

import yaml
from typing import Optional, Dict, Any, TextIO, TYPE_CHECKING

from checkov.common.parallelizer.parallel_runner import parallel_runner


from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.graph.graph_builder.consts import GraphSource
from checkov.common.images.image_referencer import Image
from checkov.common.output.record import Record
from checkov.common.output.report import Report
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.runners.base_runner import BaseRunner, filter_ignored_paths
from checkov.common.typing import _CheckResult, _EntityContext
from checkov.common.util.consts import START_LINE, END_LINE
from checkov.common.util.data_structures_utils import pickle_deepcopy
from checkov.common.util.type_forcers import convert_str_to_bool
from checkov.kubernetes.kubernetes_utils import create_check_result, get_resource_id, calculate_code_lines, \
    PARENT_RESOURCE_ID_KEY_NAME
from checkov.kubernetes.runner import Runner as K8sRunner, _get_entity_abs_path, _KubernetesContext, _KubernetesDefinitions
from checkov.kustomize.image_referencer.manager import KustomizeImageReferencerManager
from checkov.kustomize.utils import get_kustomize_version, get_kubectl_version
from checkov.runner_filter import RunnerFilter
from checkov.common.graph.checks_infra.registry import BaseRegistry
from checkov.common.typing import LibraryGraphConnector
from checkov.kubernetes.graph_builder.local_graph import KubernetesLocalGraph

if TYPE_CHECKING:
    from checkov.common.checks.base_check import BaseCheck
    from checkov.common.graph.checks_infra.base_check import BaseGraphCheck
    from checkov.kubernetes.graph_manager import KubernetesGraphManager
    from networkx import DiGraph


# Regex matching any remote resource reference that kustomize would fetch over the network.
# Covers: http(s)://, git::, ssh://, github.com/ shorthand
_REMOTE_REF = re.compile(r'^(https?://|git::|ssh://|github\.com/)', re.IGNORECASE)

# Keys in kustomization.yaml that may contain remote references
_REMOTE_REF_KEYS = ('resources', 'bases', 'components', 'crds')


def _get_kustomization_remote_refs(kustomization_dir: str) -> list[str]:
    """Return all remote resource references found in the kustomization file.

    Returns an empty list when the file cannot be read or contains no remote refs.
    """
    for filename in ('kustomization.yaml', 'kustomization.yml'):
        kustomization_path = os.path.join(kustomization_dir, filename)
        if not os.path.isfile(kustomization_path):
            continue
        try:
            with open(kustomization_path, 'r', encoding='utf-8') as f:
                doc = yaml.safe_load(f)
        except Exception:
            logging.debug(f"Could not parse {kustomization_path} for remote-ref check.", exc_info=True)
            return []

        if not isinstance(doc, dict):
            return []

        remote_refs: list[str] = []
        for key in _REMOTE_REF_KEYS:
            entries = doc.get(key) or []
            if not isinstance(entries, list):
                continue
            for entry in entries:
                if isinstance(entry, str) and _REMOTE_REF.match(entry):
                    remote_refs.append(entry)
        return remote_refs

    return []


def _get_blocked_remote_refs(kustomization_dir: str) -> list[str]:
    """Return remote refs blocked by the CHECKOV_KUSTOMIZE_ALLOWED_REMOTE_PREFIXES allowlist.

    Not set → [] (allow all). Set → returns refs not matching any allowed prefix (these are blocked).
    To block all remote refs, set the var to a non-matching value (e.g. 'none').
    """
    allowed_prefixes_raw = os.getenv("CHECKOV_KUSTOMIZE_ALLOWED_REMOTE_PREFIXES", "")
    if not allowed_prefixes_raw.strip():
        return []  # no allowlist configured → allow everything (original behaviour)

    allowed_prefixes = [p.strip().lower() for p in allowed_prefixes_raw.split(",") if p.strip()]
    remote_refs = _get_kustomization_remote_refs(kustomization_dir)
    return [ref for ref in remote_refs if not any(ref.lower().startswith(p) for p in allowed_prefixes)]


class K8sKustomizeRunner(K8sRunner):
    check_type = CheckType.KUSTOMIZE  # noqa: CCE003  # a static attribute

    def __init__(
        self,
        graph_class: type[KubernetesLocalGraph] = KubernetesLocalGraph,
        db_connector: LibraryGraphConnector | None = None,
        source: str = GraphSource.KUBERNETES,
        graph_manager: KubernetesGraphManager | None = None,
        external_registries: list[BaseRegistry] | None = None
    ) -> None:

        super().__init__(graph_class, db_connector, source, graph_manager, external_registries, CheckType.KUSTOMIZE)
        self.report_mutator_data: "dict[str, dict[str, Any]]" = {}
        self.original_root_dir: str = ''
        self.pbar.turn_off_progress_bar()

        # Allows using kustomize commands to directly edit the user's kustomization.yaml configurations
        self.checkov_allow_kustomize_file_edits = convert_str_to_bool(os.getenv("CHECKOV_ALLOW_KUSTOMIZE_FILE_EDITS",
                                                                                False))

    def set_external_data(
        self,
        definitions: _KubernetesDefinitions | None,
        context: dict[str, dict[str, Any]] | None,
        breadcrumbs: dict[str, dict[str, Any]] | None,
        report_mutator_data: dict[str, dict[str, Any]] | None = None,
        **kwargs: Any,
    ) -> None:
        super().set_external_data(definitions, context, breadcrumbs)
        self.report_mutator_data = report_mutator_data or {}

    def set_report_mutator_data(self, report_mutator_data: Optional[Dict[str, Dict[str, Any]]]) -> None:
        self.report_mutator_data = report_mutator_data or {}

    @staticmethod
    def get_kustomize_resource_id(realKustomizeEnvMetadata: dict[str, Any], resource_id: str) -> str:
        if 'overlay' in realKustomizeEnvMetadata["type"]:
            kustomizeResourceID = f'{realKustomizeEnvMetadata["type"]}:{str(realKustomizeEnvMetadata["overlay_name"])}:{resource_id}'
        else:
            kustomizeResourceID = f'{realKustomizeEnvMetadata["type"]}:{resource_id}'
        return kustomizeResourceID

    def mutate_kubernetes_results(
        self,
        results: dict[BaseCheck, _CheckResult],
        report: Report,
        k8_file: str,
        k8_file_path: str,
        file_abs_path: str,
        entity_conf: dict[str, Any],
        variable_evaluations: dict[str, Any],
        root_folder: str | None = None
    ) -> Report:
        # Moves report generation logic out of checkov.kubernetes.runner.run() def.
        # Allows us to overriding report file information for "child" frameworks such as Kustomize, Helm
        # Where Kubernetes CHECKS are needed, but the specific file references are to another framework for the user output (or a mix of both).
        if not self.context:
            # this shouldn't happen
            logging.error("context was not set correctly")
            return report

        kustomize_metadata = self.report_mutator_data['kustomizeMetadata'],
        kustomize_file_mappings = self.report_mutator_data['kustomizeFileMappings']
        for check, check_result in results.items():
            resource_id = get_resource_id(entity_conf)
            if not resource_id:
                logging.error(f"Couldn't get resource ID for {entity_conf}")
                continue

            entity_context = self.context[k8_file][resource_id]

            if file_abs_path not in kustomize_file_mappings:
                logging.warning(f"couldn't find {file_abs_path} path in kustomizeFileMappings")
                continue

            realKustomizeEnvMetadata = kustomize_metadata[0][kustomize_file_mappings[file_abs_path]]
            kustomizeResourceID = self.get_kustomize_resource_id(realKustomizeEnvMetadata, resource_id)

            external_run_indicator = "Bc"
            file_path = realKustomizeEnvMetadata['filePath']

            caller_file_path = None
            caller_file_line_range = None

            # means this scan originated in the platform
            if type(self.graph_manager).__name__.startswith(external_run_indicator):
                absolute_file_path = file_abs_path
            else:
                absolute_file_path = realKustomizeEnvMetadata['filePath']
                # Fix file path to repo relative path
                if self.original_root_dir:
                    repo_dir = str(pathlib.Path(self.original_root_dir).resolve())

                    if realKustomizeEnvMetadata['filePath'].startswith(repo_dir):
                        file_path = realKustomizeEnvMetadata['filePath'][len(repo_dir):]

            if self.checkov_allow_kustomize_file_edits:
                caller_resource_id = resource_id
                if PARENT_RESOURCE_ID_KEY_NAME in entity_conf:
                    caller_resource_id = entity_conf[PARENT_RESOURCE_ID_KEY_NAME]
                caller_file_line_range, caller_file_path = self._get_caller_file_info(entity_context, k8_file,
                                                                                      k8_file_path,
                                                                                      resource_id,
                                                                                      caller_resource_id,
                                                                                      root_folder)
            code_lines = entity_context.get("code_lines")
            file_line_range = self.line_range(code_lines)

            record = Record(
                check_id=check.id, bc_check_id=check.bc_id, check_name=check.name,
                check_result=check_result, code_block=code_lines,
                file_path=file_path, file_line_range=file_line_range,
                caller_file_path=caller_file_path, caller_file_line_range=caller_file_line_range,
                resource=kustomizeResourceID, evaluations=variable_evaluations,
                check_class=check.__class__.__module__, file_abs_path=absolute_file_path, severity=check.severity)
            record.set_guideline(check.guideline)
            report.add_record(record=record)

        return report

    def _get_caller_file_info(self, entity_context: _EntityContext, k8_file: str, k8_file_path: str, resource_id: str,
                              caller_resource_id: str, root_folder: str | None) -> tuple[tuple[int, int] | None, str | None]:
        origin_relative_path = entity_context.get('origin_relative_path')
        if origin_relative_path is None:
            return None, None
        k8s_file_dir = pathlib.Path(k8_file_path).parent
        raw_file_path = k8s_file_dir / origin_relative_path
        caller_file_path = self._get_caller_file_path(k8s_file_dir, origin_relative_path, raw_file_path)
        if root_folder is None:
            return None, caller_file_path
        caller_file_line_range = self._get_caller_line_range(root_folder, k8_file, origin_relative_path,
                                                             resource_id, caller_resource_id)
        return caller_file_line_range, caller_file_path

    @staticmethod
    def _get_caller_file_path(k8s_file_dir: pathlib.Path, origin_relative_path: str, raw_file_path: pathlib.Path)\
            -> str:
        """
        Creates the correct file path based on the collection of metadata locations we have.

        Example for expected input:
            - k8s_fil_dir - Path('/resources/image_referencer/overlays/prod')
            - origin_relative_path - '../../base/deployment.yaml'
            - raw_file_path - Path('/resources/image_referencer/overlays/prod/../../base/deployment.yaml')
        """
        amount_of_parents = str.count(origin_relative_path, '..')
        directory_prefix_path = k8s_file_dir
        if amount_of_parents == 0:
            # In case we don't have any relative paths, we need to remove the first directory parent
            # (as the first directory is the same one of the kustomization.yaml file)
            directory_prefix_path = k8s_file_dir.parent
        elif amount_of_parents and len(k8s_file_dir.parents) >= amount_of_parents:
            directory_prefix_path = k8s_file_dir.parents[amount_of_parents - 1]

        directory_prefix = str(directory_prefix_path)
        resolved_path = str(raw_file_path.resolve())
        # Make sure the resolved path starts with the root folder, as pathlib.Path.resolve() might change it
        if directory_prefix in resolved_path and not resolved_path.startswith(directory_prefix):
            resolved_path = K8sKustomizeRunner._remove_extra_path_parts(resolved_path, directory_prefix)

        return resolved_path[len(str(directory_prefix)):]

    @staticmethod
    def _remove_extra_path_parts(resolved_path: str, prefix: str) -> str:
        """
        Some pathlib paths can "add" extra arguments at the beginning after running `Path.resolve()`.
        For example, running `Path('/var/example.txt').resolve` might result in `/<not-existent-dir>/var/example.txt`.
        The purpose of this function is to remove any unintentional additions like this one.
        """
        resolved_path_parts = resolved_path.split(prefix)
        if len(resolved_path_parts) > 1:
            resolved_path = f'{prefix}{"".join(resolved_path_parts[1:])}'
        else:
            resolved_path = f'{prefix}{"".join(resolved_path_parts)}'
        return resolved_path

    def _get_caller_line_range(self, root_folder: str, k8_file: str, origin_relative_path: str,
                               resource_id: str, caller_resource_id: str) -> tuple[int, int] | None:
        raw_caller_directory = (pathlib.Path(k8_file.lstrip(os.path.sep)).parent /
                                pathlib.Path(origin_relative_path.lstrip(os.path.sep)).parent)
        caller_directory = str(pathlib.Path(f'{os.path.sep}{raw_caller_directory}').resolve())
        caller_directory = K8sKustomizeRunner._remove_extra_path_parts(caller_directory, root_folder)
        file_ending = pathlib.Path(origin_relative_path).suffix
        caller_file_path = f'{str(pathlib.Path(caller_directory) / caller_resource_id.replace(".", "-"))}{file_ending}'

        if caller_file_path not in self.definitions:
            return None

        caller_resource = None
        for resource in self.definitions[caller_file_path]:
            _resource_id = get_resource_id(resource)
            if _resource_id == resource_id:
                caller_resource = resource
                break

        if caller_resource is None:
            return None

        if caller_file_path not in self.definitions_raw:
            # As we cannot calculate better lines with the `calculate_code_lines` without the raw code,
            # we can use the existing info in the resource
            return caller_resource[START_LINE], caller_resource[END_LINE]

        raw_caller_resource = self.definitions_raw[caller_file_path]

        caller_raw_start_line = caller_resource[START_LINE]
        caller_raw_end_line = min(caller_resource[END_LINE], len(raw_caller_resource))

        _, caller_start_line, caller_end_line = calculate_code_lines(raw_caller_resource, caller_raw_start_line,
                                                                     caller_raw_end_line)
        return caller_start_line, caller_end_line

    def line_range(self, code_lines: list[tuple[int, str]]) -> list[int]:
        num_of_lines = len(code_lines)
        file_line_range = [0, 0]
        if num_of_lines > 0:
            first_line, code = code_lines[0]
            last_line, code = code_lines[num_of_lines - 1]
            file_line_range = [first_line, last_line]
        return file_line_range

    def mutate_kubernetes_graph_results(
        self, root_folder: str | None, runner_filter: RunnerFilter, report: Report, checks_results: dict[BaseGraphCheck, list[_CheckResult]]
    ) -> Report:
        # Moves report generation logic out of run() method in Runner class.
        # Allows function overriding of a much smaller function than run() for other "child" frameworks such as Kustomize, Helm
        # Where Kubernetes CHECKS are needed, but the specific file references are to another framework for the user output (or a mix of both).
        if not self.context:
            if self.context is None:
                # this shouldn't happen
                logging.error("Context for Kustomize runner was not set")
            return report

        kustomize_metadata = self.report_mutator_data['kustomizeMetadata'],
        kustomize_file_mappings = self.report_mutator_data['kustomizeFileMappings']

        for check, check_results in checks_results.items():
            for check_result in check_results:
                entity = check_result["entity"]
                entity_file_path: str = entity[CustomAttributes.FILE_PATH]
                entity_file_abs_path: str = _get_entity_abs_path(root_folder, entity_file_path)
                entity_id: str = entity[CustomAttributes.ID]
                entity_context = super().get_entity_context(entity=entity, entity_file_path=entity_file_path)

                if entity_file_abs_path in kustomize_file_mappings:
                    realKustomizeEnvMetadata = kustomize_metadata[0][kustomize_file_mappings[entity_file_abs_path]]
                    if 'overlay' in realKustomizeEnvMetadata["type"]:
                        kustomizeResourceID = f'{realKustomizeEnvMetadata["type"]}:{str(realKustomizeEnvMetadata["overlay_name"])}:{entity_id}'
                    else:
                        kustomizeResourceID = f'{realKustomizeEnvMetadata["type"]}:{entity_id}'
                else:
                    logging.warning(f"couldn't find {entity_file_abs_path} path in kustomizeFileMappings")
                    continue

                caller_file_path = None
                caller_file_line_range = None
                if self.checkov_allow_kustomize_file_edits:
                    caller_resource_id = entity_id
                    if PARENT_RESOURCE_ID_KEY_NAME in entity:
                        caller_resource_id = entity[PARENT_RESOURCE_ID_KEY_NAME]
                    caller_file_line_range, caller_file_path = self._get_caller_file_info(entity_context,
                                                                                          entity_file_path,
                                                                                          entity_file_path,
                                                                                          entity_id,
                                                                                          caller_resource_id,
                                                                                          root_folder)
                code_lines = entity_context["code_lines"]
                file_line_range = self.line_range(code_lines)

                clean_check_result = create_check_result(check_result=check_result, entity_context=entity_context, check_id=check.id)

                record = Record(
                    check_id=check.id,
                    check_name=check.name,
                    check_result=clean_check_result,
                    code_block=code_lines,
                    file_path=realKustomizeEnvMetadata['filePath'],
                    file_line_range=file_line_range,
                    caller_file_path=caller_file_path,
                    caller_file_line_range=caller_file_line_range,
                    resource=kustomizeResourceID,  # entity.get(CustomAttributes.ID),
                    evaluations={},
                    check_class=check.__class__.__module__,
                    file_abs_path=entity_file_abs_path,
                    severity=check.severity
                )
                record.set_guideline(check.guideline)
                report.add_record(record=record)

        return report

    def get_image_report(self, root_folder: str | None, runner_filter: RunnerFilter) -> Report | None:
        if not self.graph_manager:
            return None
        return self.check_container_image_references(
            graph_connector=self.graph_manager.get_reader_endpoint(),
            root_path=self.original_root_dir,
            runner_filter=runner_filter,
        )

    def extract_images(
            self,
            graph_connector: DiGraph | None = None,
            definitions: None = None,
            definitions_raw: dict[str, list[tuple[int, str]]] | None = None
    ) -> list[Image]:
        if not graph_connector:
            # should not happen
            return []

        manager = KustomizeImageReferencerManager(graph_connector=graph_connector, report_mutator_data=self.report_mutator_data)
        images = manager.extract_images_from_resources()

        return images


class Runner(BaseRunner[_KubernetesDefinitions, _KubernetesContext, "KubernetesGraphManager"]):
    kustomize_command = 'kustomize'  # noqa: CCE003  # a static attribute
    kubectl_command = 'kubectl'  # noqa: CCE003  # a static attribute
    check_type = CheckType.KUSTOMIZE  # noqa: CCE003  # a static attribute
    system_deps = True  # noqa: CCE003  # a static attribute
    kustomizeSupportedFileTypes = ('kustomization.yaml', 'kustomization.yml')  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        super().__init__(file_names=Runner.kustomizeSupportedFileTypes)
        self.potentialBases: "list[str]" = []
        self.potentialOverlays: "list[str]" = []
        self.kustomizeProcessedFolderAndMeta: "dict[str, dict[str, str]]" = {}
        self.kustomizeFileMappings: "dict[str, str]" = {}
        self.templateRendererCommand: str | None = None
        self.target_folder_path = ''

        self.checkov_allow_kustomize_file_edits = convert_str_to_bool(os.getenv("CHECKOV_ALLOW_KUSTOMIZE_FILE_EDITS",
                                                                                False))

    def get_k8s_target_folder_path(self) -> str:
        return self.target_folder_path

    def get_kustomize_metadata(self) -> dict[str, dict[str, Any]]:
        return {'kustomizeMetadata': self.kustomizeProcessedFolderAndMeta,
                'kustomizeFileMappings': self.kustomizeFileMappings}

    def _parseKustomization(self, kustomize_dir: str) -> dict[str, Any]:
        # We may have multiple results for "kustomization.yaml" files. These could be:
        # - Base and Environment (overlay) DIR's for the same kustomize-powered deployment
        # - OR, Multiple different Kustomize-powered deployments
        # - OR, a mixture of the two.
        # We need parse some of the Kustomization.yaml files to work out which
        # This is so we can provide "Environment" information back to the user as part of the checked resource name/description.
        # TODO: We could also add a --kustomize-environment option so we only scan certain overlay names (prod, test etc) useful in CI.
        yaml_path = os.path.join(kustomize_dir, "kustomization.yaml")
        yml_path = os.path.join(kustomize_dir, "kustomization.yml")
        if os.path.isfile(yml_path):
            kustomization_path = yml_path
        elif os.path.isfile(yaml_path):
            kustomization_path = yaml_path
        else:
            return {}

        with open(kustomization_path, 'r') as kustomization_file:
            metadata: dict[str, Any] = {}
            try:
                file_content = yaml.safe_load(kustomization_file)
            except yaml.YAMLError:
                logging.info(f"Failed to load Kustomize metadata from {kustomization_path}.", exc_info=True)
                return {}

            if not isinstance(file_content, dict):
                return {}

            if 'resources' in file_content and file_content['resources'] is not None:
                resources = file_content['resources']

                # We can differentiate between "overlays" and "bases" based on if the `resources` refers to a directory,
                # which represents an "overlay", or only files which represents a "base"
                resources_representing_directories = [r for r in resources if pathlib.Path(r).suffix == '']
                if resources_representing_directories:
                    logging.debug(
                        f"Kustomization contains resources: section with directories. Likely an overlay/env."
                        f" {kustomization_path}")
                    metadata['type'] = "overlay"
                    metadata['referenced_bases'] = resources_representing_directories
                else:
                    logging.debug(f"Kustomization contains resources: section with only files (no dirs). Likely a base."
                                  f" {kustomization_path}")
                    metadata['type'] = "base"

            elif 'patchesStrategicMerge' in file_content:
                logging.debug(f"Kustomization contains patchesStrategicMerge: section. Likely an overlay/env. {kustomization_path}")
                metadata['type'] = "overlay"
                if 'bases' in file_content:
                    metadata['referenced_bases'] = file_content['bases']

            elif 'bases' in file_content:
                logging.debug(f"Kustomization contains bases: section. Likely an overlay/env. {kustomization_path}")
                metadata['type'] = "overlay"
                metadata['referenced_bases'] = file_content['bases']

            metadata['fileContent'] = file_content
            metadata['filePath'] = f"{kustomization_path}"
            if metadata.get('type') == "base":
                self.potentialBases.append(metadata['filePath'])

            if metadata.get('type') == "overlay":
                self.potentialOverlays.append(metadata['filePath'])

        return metadata

    def check_system_deps(self) -> str | None:
        # Ensure local system dependencies are available and of the correct version.
        # Returns framework names to skip if deps **fail** (ie, return None for a successful deps check).
        logging.info(f"Checking necessary system dependencies for {self.check_type} checks.")

        if shutil.which(self.kubectl_command) is not None:
            kubectl_version = get_kubectl_version(kubectl_command=self.kubectl_command)
            if kubectl_version and kubectl_version >= 1.14:
                logging.info(f"Found working version of {self.check_type} dependency {self.kubectl_command}: {kubectl_version}")
                self.templateRendererCommand = self.kubectl_command
                return None
            else:
                return self.check_type
        elif shutil.which(self.kustomize_command) is not None:
            kustomize_version = get_kustomize_version(kustomize_command=self.kustomize_command)
            if kustomize_version:
                logging.info(
                    f"Found working version of {self.check_type} dependency {self.kustomize_command}: {kustomize_version}"
                )
                self.templateRendererCommand = self.kustomize_command
                return None
            else:
                return self.check_type
        else:
            logging.info(f"Could not find usable tools locally to process {self.check_type} checks. Framework will be disabled for this run.")
            return self.check_type

    def _handle_overlay_case(self, file_path: str,
                             kustomizeProcessedFolderAndMeta: dict[str, dict[str, Any]] | None = None) -> None:
        if kustomizeProcessedFolderAndMeta is None:
            kustomizeProcessedFolderAndMeta = self.kustomizeProcessedFolderAndMeta
        for parent in pathlib.Path(file_path).parents:
            for potentialBase in self.potentialBases:
                pathlib_base_object = pathlib.Path(potentialBase)
                potential_base_path = pathlib_base_object.parents[1]
                if parent == potential_base_path.resolve():
                    kustomizeProcessedFolderAndMeta[file_path]['calculated_bases'] = str(pathlib_base_object.parent)
        try:
            relativeToFullPath = f"{file_path}/{kustomizeProcessedFolderAndMeta[file_path]['referenced_bases'][0]}"
            if pathlib.Path(kustomizeProcessedFolderAndMeta[file_path]['calculated_bases']) == pathlib.Path(relativeToFullPath).resolve():
                kustomizeProcessedFolderAndMeta[file_path]['validated_base'] = str(pathlib.Path(kustomizeProcessedFolderAndMeta[file_path]['calculated_bases']))
                checkov_kustomize_env_name_by_path = str(pathlib.Path(file_path).relative_to(pathlib.Path(kustomizeProcessedFolderAndMeta[file_path]['calculated_bases']).parent))
                kustomizeProcessedFolderAndMeta[file_path]['overlay_name'] = checkov_kustomize_env_name_by_path
                logging.debug(f"Overlay based on {kustomizeProcessedFolderAndMeta[file_path]['validated_base']}, naming overlay {checkov_kustomize_env_name_by_path} for Checkov Results.")
            else:
                checkov_kustomize_env_name_by_path = pathlib.Path(file_path).stem
                kustomizeProcessedFolderAndMeta[file_path]['overlay_name'] = checkov_kustomize_env_name_by_path
                logging.debug(f"Could not confirm base dir for Kustomize overlay/env. Using {checkov_kustomize_env_name_by_path} for Checkov Results.")

        except KeyError:
            checkov_kustomize_env_name_by_path = pathlib.Path(file_path).stem
            kustomizeProcessedFolderAndMeta[file_path]['overlay_name'] = checkov_kustomize_env_name_by_path
            logging.debug(f"Could not confirm base dir for Kustomize overlay/env. Using {checkov_kustomize_env_name_by_path} for Checkov Results.")

    @staticmethod
    def _get_parsed_output(
        file_path: str, extract_dir: str, output: str, shared_kustomize_file_mappings: dict[str, str]
    ) -> TextIO | None:
        cur_source_file = None
        cur_writer = None
        last_line_dashes = False
        line_num = 1
        file_num = 0

        # page-to-file parser from helm framework works well, but we expect the file to start with --- in this case from Kustomize.
        output = "---\n" + output
        reader = io.StringIO(output)
        for s in reader:
            s = s.rstrip()
            if s == '---':
                last_line_dashes = True
                continue

            if last_line_dashes:
                # The next line should contain a "apiVersion" line for the next Kubernetes manifest
                # So we will close the old file, open a new file, and write the dashes from last iteration plus this line
                source = file_num
                file_num += 1
                if source != cur_source_file:
                    if cur_writer:
                        # Here we are about to close a "complete" file. The function will validate it looks like a K8S manifest before continuing.
                        Runner._curWriterValidateStoreMapAndClose(cur_writer, file_path, shared_kustomize_file_mappings)
                    parent = os.path.dirname(os.path.join(extract_dir, str(source)))
                    os.makedirs(parent, exist_ok=True)
                    cur_source_file = source
                    cur_writer = open(os.path.join(extract_dir, str(source)), 'a')
                if cur_writer:
                    cur_writer.write('---' + os.linesep)
                    cur_writer.write(s + os.linesep)

                last_line_dashes = False
            else:
                if not cur_writer:
                    continue
                else:
                    cur_writer.write(s + os.linesep)
            line_num += 1
        return cur_writer

    def _get_kubectl_output(self, filePath: str, template_renderer_command: str, source_type: str | None) -> bytes | None:
        # Template out the Kustomizations to Kubernetes YAML
        if template_renderer_command == "kubectl":
            template_render_command_options = "kustomize"
        elif template_renderer_command == "kustomize":
            template_render_command_options = "build"
        else:
            logging.error(f"Template renderer command has an invalid value: {template_renderer_command}")
            return None

        # Default = allow all. Set CHECKOV_KUSTOMIZE_ALLOWED_REMOTE_PREFIXES
        # to a comma-separated allowlist of trusted URL prefixes to block everything else.
        blocked_refs = _get_blocked_remote_refs(filePath)
        if blocked_refs:
            logging.warning(
                f"Skipping kustomize build for {filePath}: {len(blocked_refs)} remote reference(s) "
                f"not in CHECKOV_KUSTOMIZE_ALLOWED_REMOTE_PREFIXES allowlist."
            )
            return None

        add_origin_annotations_return_code = None

        if self.checkov_allow_kustomize_file_edits:
            add_origin_annotations_command = 'kustomize edit add buildmetadata originAnnotations'
            add_origin_annotations_return_code = subprocess.run(add_origin_annotations_command.split(' '),  # nosec
                                                                cwd=filePath).returncode

        full_command = f'{template_renderer_command} {template_render_command_options}'
        proc = subprocess.Popen(full_command.split(' '), cwd=filePath, stdout=subprocess.PIPE, stderr=subprocess.PIPE)  # nosec
        output, _ = proc.communicate()

        if self.checkov_allow_kustomize_file_edits and add_origin_annotations_return_code == 0:
            # If the return code is not 0, we didn't add the new buildmetadata field, so we shouldn't remove it
            remove_origin_annotaions = 'kustomize edit remove buildmetadata originAnnotations'
            subprocess.run(remove_origin_annotaions.split(' '), cwd=filePath)  # nosec

        logging.info(
            f"Ran kubectl to build Kustomize output. DIR: {filePath}. TYPE: {source_type}.")
        return output

    @staticmethod
    def _get_env_or_base_path_prefix(
        file_path: str, kustomize_processed_folder_and_meta: dict[str, dict[str, Any]]
    ) -> str | None:
        env_or_base_path_prefix = None
        if kustomize_processed_folder_and_meta[file_path].get('type') == "overlay":
            if 'calculated_bases' not in kustomize_processed_folder_and_meta[file_path]:
                logging.debug(f"Kustomize: Overlay with unknown base. User may have specified overlay dir directly. {file_path}")
                env_or_base_path_prefix = ""
            else:
                base_path_parts = pathlib.Path(kustomize_processed_folder_and_meta[file_path]['calculated_bases']).parts
                most_significant_base_path = f"/{base_path_parts[-3]}/{base_path_parts[-2]}"
                env_or_base_path_prefix = f"{most_significant_base_path}/{kustomize_processed_folder_and_meta[file_path]['overlay_name']}"

        elif kustomize_processed_folder_and_meta[file_path].get('type') == "base":
            # Validated base last three parents as a path
            base_path_parts = pathlib.Path(kustomize_processed_folder_and_meta[file_path]['filePath']).parts
            most_significant_base_path = f"/{base_path_parts[-4]}/{base_path_parts[-3]}/{base_path_parts[-2]}"
            env_or_base_path_prefix = most_significant_base_path

        return env_or_base_path_prefix

    def get_binary_output_from_directory(
            self,
            file_path: str,
            template_renderer_command: str,
    ) -> tuple[bytes, str] | tuple[None, None]:
        kustomizeProcessedFolderAndMeta = {file_path: self._parseKustomization(file_path)}
        if kustomizeProcessedFolderAndMeta[file_path].get('type') == 'overlay':
            self._handle_overlay_case(file_path, kustomizeProcessedFolderAndMeta)
        return self.get_binary_output(file_path, kustomizeProcessedFolderAndMeta, template_renderer_command)

    def get_binary_output(
        self,
        file_path: str,
        kustomize_processed_folder_and_meta: dict[str, dict[str, Any]],
        template_renderer_command: str,
    ) -> tuple[bytes, str] | tuple[None, None]:
        source_type = kustomize_processed_folder_and_meta[file_path].get('type')
        logging.debug(f"Kustomization at {file_path} likely a {source_type}")
        try:
            output = self._get_kubectl_output(file_path, template_renderer_command, source_type)
            if output is None:
                return None, None

            return output, file_path
        except Exception:
            logging.warning(f"Error building Kustomize output at dir: {file_path}.", exc_info=True)
            return None, None

    @staticmethod
    def _parse_output(
        output: bytes,
        file_path: str,
        kustomize_processed_folder_and_meta: dict[str, dict[str, Any]],
        target_folder_path: str,
        shared_kustomize_file_mappings: dict[str, str],
    ) -> None:
        env_or_base_path_prefix = Runner._get_env_or_base_path_prefix(file_path, kustomize_processed_folder_and_meta)
        if env_or_base_path_prefix is None:
            logging.warning(f"env_or_base_path_prefix is None, filePath: {file_path}", exc_info=True)
            return

        extract_dir = target_folder_path + env_or_base_path_prefix
        os.makedirs(extract_dir, exist_ok=True)

        logging.debug(f"Kustomize: Temporary directory for {file_path} at {extract_dir}")
        output_str = output.decode("utf-8")
        cur_writer = Runner._get_parsed_output(file_path, extract_dir, output_str, shared_kustomize_file_mappings)
        if cur_writer:
            Runner._curWriterValidateStoreMapAndClose(cur_writer, file_path, shared_kustomize_file_mappings)

    def _run_kustomize_parser(
        self,
        file_path: str,
        shared_kustomize_file_mappings: dict[str, str],
        kustomize_processed_folder_and_meta: dict[str, dict[str, Any]],
        template_renderer_command: str,
        target_folder_path: str,
    ) -> None:
        output, _ = self.get_binary_output(file_path, kustomize_processed_folder_and_meta, template_renderer_command)
        if not output:
            return
        Runner._parse_output(output, file_path, kustomize_processed_folder_and_meta, target_folder_path, shared_kustomize_file_mappings)

    def run_kustomize_to_k8s(
        self, root_folder: str | None, files: list[str] | None, runner_filter: RunnerFilter
    ) -> None:
        kustomize_dirs = find_kustomize_directories(root_folder, files, runner_filter.excluded_paths)
        if not kustomize_dirs:
            # nothing to process
            return

        for kustomize_dir in kustomize_dirs:
            self.kustomizeProcessedFolderAndMeta[kustomize_dir] = self._parseKustomization(kustomize_dir)
        self.target_folder_path = tempfile.mkdtemp()
        for file_path in self.kustomizeProcessedFolderAndMeta:
            if self.kustomizeProcessedFolderAndMeta[file_path].get('type') == 'overlay':
                self._handle_overlay_case(file_path)

        if platform.system() == 'Windows':
            if not self.templateRendererCommand:
                logging.error("The 'templateRendererCommand' was not set correctly")
                return

            shared_kustomize_file_mappings: dict[str, str] = {}
            for file_path in self.kustomizeProcessedFolderAndMeta:
                self._run_kustomize_parser(
                    file_path=file_path,
                    shared_kustomize_file_mappings=shared_kustomize_file_mappings,
                    kustomize_processed_folder_and_meta=self.kustomizeProcessedFolderAndMeta,
                    template_renderer_command=self.templateRendererCommand,
                    target_folder_path=self.target_folder_path,
                )
            self.kustomizeFileMappings = shared_kustomize_file_mappings
            return

        manager = multiprocessing.Manager()
        # make sure we have new dict
        shared_kustomize_file_mappings = pickle_deepcopy(manager.dict())  # type:ignore[arg-type]  # works with DictProxy
        shared_kustomize_file_mappings.clear()

        items = [
            (
                filePath,
                shared_kustomize_file_mappings,
                self.kustomizeProcessedFolderAndMeta,
                self.templateRendererCommand,
                self.target_folder_path,
            )
            for filePath in self.kustomizeProcessedFolderAndMeta
        ]
        list(parallel_runner.run_function(self._run_kustomize_parser, items))

        self.kustomizeFileMappings = dict(shared_kustomize_file_mappings)

    def run(
        self,
        root_folder: str | None,
        external_checks_dir: list[str] | None = None,
        files: list[str] | None = None,
        runner_filter: RunnerFilter | None = None,
        collect_skip_comments: bool = True,
    ) -> Report | list[Report]:
        runner_filter = runner_filter or RunnerFilter()
        if not runner_filter.show_progress_bar:
            self.pbar.turn_off_progress_bar()

        self.run_kustomize_to_k8s(root_folder, files, runner_filter)
        report: "Report | list[Report]" = Report(self.check_type)

        if not self.kustomizeProcessedFolderAndMeta:
            # nothing to process
            return report

        target_dir = ""
        try:
            k8s_runner = K8sKustomizeRunner()
            # k8s_runner.run() will kick off both CKV_ and CKV2_ checks and return a merged results object.
            target_dir = self.get_k8s_target_folder_path()
            k8s_runner.report_mutator_data = self.get_kustomize_metadata()
            if root_folder:
                k8s_runner.original_root_dir = root_folder

            # the returned report can be a list of reports, which also includes an SCA image report
            report = k8s_runner.run(target_dir, external_checks_dir=external_checks_dir, runner_filter=runner_filter)
            self.graph_manager = k8s_runner.graph_manager
            logging.debug(f"Successfully ran k8s scan on Kustomization templated files in tmp scan dir : {target_dir}")

            shutil.rmtree(target_dir)

        except Exception:
            logging.warning("Failed to run Kubernetes runner", exc_info=True)
            with tempfile.TemporaryDirectory() as save_error_dir:
                logging.debug(
                    f"Error running k8s scan on Scan dir: {target_dir}. Saved context dir: {save_error_dir}")
                shutil.move(target_dir, save_error_dir)

        return report

    @staticmethod
    def _curWriterValidateStoreMapAndClose(
        cur_writer: TextIO, file_path: str, shared_kustomize_file_mappings: dict[str, str]
    ) -> None:
        currentFileName = cur_writer.name
        cur_writer.close()
        # Now we have a complete k8s manifest as we closed the writer, and it's temporary file name (currentFileName) plus the original file templated out (FilePath)
        # Rename them to useful information from the K8S metadata before conting.
        # Then keep a mapping of template files to original kustomize repo locations for use with Checkov output later.
        try:
            with open(currentFileName) as f:
                currentYamlObject = yaml.safe_load(f)
                # Validate we have a K8S manifest

                if "apiVersion" in currentYamlObject:
                    itemName = []
                    itemName.append(currentYamlObject['kind'])
                    if 'namespace' in currentYamlObject['metadata']:
                        itemName.append(currentYamlObject['metadata']['namespace'])
                    else:
                        itemName.append("default")
                    if 'name' in currentYamlObject['metadata']:
                        itemName.append(currentYamlObject['metadata']['name'])
                    else:
                        itemName.append("noname")

                    filename = f"{'-'.join(itemName)}.yaml"
                    newFullPathFilename = str(pathlib.Path(currentFileName).parent / filename)
                    os.rename(currentFileName, newFullPathFilename)
                    shared_kustomize_file_mappings[newFullPathFilename] = file_path
                else:
                    raise Exception(f'Not a valid Kubernetes manifest (no apiVersion) while parsing Kustomize template: {file_path}. Templated output: {currentFileName}.')
        except IsADirectoryError:
            pass


def find_kustomize_directories(
    root_folder: str | None, files: list[str] | None, excluded_paths: list[str]
) -> list[str]:
    kustomize_directories = []
    if not excluded_paths:
        excluded_paths = []
    if files:
        logging.info('Running with --file argument; file must be a kustomization.yaml file')
        for file in files:
            if os.path.basename(file) in Runner.kustomizeSupportedFileTypes:
                kustomize_directories.append(os.path.dirname(file))

    if root_folder:
        for root, d_names, f_names in os.walk(root_folder):
            filter_ignored_paths(root, d_names, excluded_paths)
            filter_ignored_paths(root, f_names, excluded_paths)
            kustomize_directories.extend(
                os.path.abspath(root) for x in f_names if x in Runner.kustomizeSupportedFileTypes
            )

    return kustomize_directories



from pathlib import Path

import pytest
from pytest_mock import MockerFixture

import os
from checkov.kustomize.runner import Runner
from checkov.runner_filter import RunnerFilter
from tests.graph_utils.utils import GRAPH_FRAMEWORKS
from tests.kustomize.utils import kustomize_exists


def get_kustomize_summary(mocker: MockerFixture, graph_framework, scan_dir_path):
    dir_rel_path = os.path.realpath(scan_dir_path).replace('\\', '/')

    runner_filter = RunnerFilter(framework=["kustomize"], checks=["CKV2_K8S_6"])

    mocker.patch.dict("os.environ", {"CHECKOV_GRAPH_FRAMEWORK": graph_framework})

    runner = Runner()
    runner.templateRendererCommand = "kustomize"
    runner.templateRendererCommandOptions = "build"

    report = runner.run(root_folder=dir_rel_path, runner_filter=runner_filter, external_checks_dir=None)

    summary = report.get_summary()

    return summary


@pytest.mark.skipif(not kustomize_exists(), reason="kustomize not installed")
@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_runner(mocker: MockerFixture, graph_framework):
    scan_dir_path = Path(__file__).parent / "resources" / "example_checks"
    summary = get_kustomize_summary(mocker=mocker, graph_framework=graph_framework, scan_dir_path=scan_dir_path)

    assert summary["passed"] == 1
    assert summary["failed"] == 1
    assert summary["skipped"] == 1
    assert summary["parsing_errors"] == 0


@pytest.mark.skipif(not kustomize_exists(), reason="kustomize not installed")
@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_empty_resources(mocker: MockerFixture, graph_framework):
    scan_dir_path = Path(__file__).parent / "resources" / "empty_resources"

    summary = get_kustomize_summary(mocker=mocker, graph_framework=graph_framework, scan_dir_path=scan_dir_path)

    assert summary["passed"] == 0
    assert summary["failed"] == 0
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0



import re

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.module.base_module_check import BaseModuleCheck

MODULE_GIT_VERSION_PATTERN = re.compile(r"git::https?:\/\/[^\/]+\/.+.git\?ref=(\b[0-9a-f]{5,40}\b)")


class ModuleSourceHashCheck(BaseModuleCheck):
    def __init__(self):
        name = "Ensure module is immutable using commit hash"
        id = "CKV_TF_MODULE_2"
        supported_resources = ['module']
        categories = []
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_module_conf(self, conf):
        """
        Some test for module source
        :param conf: module call
        :return: <CheckResult>
        """

        source = conf.get('source', [])
        if not source:
            # source is using latest or tagged version
            return CheckResult.FAILED
        else:
            if MODULE_GIT_VERSION_PATTERN.match(source[0]):
                # immutable source is being used
                return CheckResult.PASSED
            # non immutable source is used
            return CheckResult.FAILED


scanner = ModuleSourceHashCheck()



from aws_cdk import core
from aws_cdk import aws_cloudfront as cloudfront

class CloudFrontDistributionStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create a CloudFront distribution
        distribution = cloudfront.CfnDistribution(
            self,
            "MyCloudFrontDistribution",
            distribution_config={
                "defaultCacheBehavior": {
                    # Configure your cache behavior as needed
                },
                "enabled": True,
            }
        )

app = core.App()
CloudFrontDistributionStack(app, "CloudFrontDistributionStack")
app.synth()


import unittest
from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.terraform.runner import Runner
from checkov.terraform.checks.data.registry import data_registry

class TestExternalData(unittest.TestCase):
    def test(self):
        test_files_dir = Path(__file__).parent / "example_external_data"
        external_check_dir = Path(__file__).parent / "external_check"

        runner = Runner()
        report = runner.run(root_folder=test_files_dir, runner_filter=RunnerFilter(checks=["CKV_TF_DATA_EXTERNAL_1"]),
                            external_checks_dir=[external_check_dir])
        summary = report.get_summary()
        print(data_registry)
        self.assertEqual(summary["passed"], 0)
        self.assertEqual(summary["failed"], 1)
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)
        check = next(c for c in data_registry.checks["external"] if c.id == "CKV_TF_DATA_EXTERNAL_1")
        data_registry.checks["external"].remove(check)


if __name__ == "__main__":
    unittest.main()



import os
import unittest
import pytest

from checkov.circleci_pipelines.runner import Runner
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.runner_filter import RunnerFilter
from checkov.circleci_pipelines.registry import registry


class TestRunnerValid(unittest.TestCase):

    def test_registry_has_type(self):
        self.assertEqual(registry.report_type, CheckType.CIRCLECI_PIPELINES)

    def test_runner(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources")
        runner = Runner()
        checks = []
        report = runner.run(
            root_folder=valid_dir_path,
            runner_filter=RunnerFilter(framework=['circleci_pipelines'], checks=checks)
        )
        self.assertEqual(len(report.failed_checks), 13)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(len(report.passed_checks), 32)
        self.assertEqual(report.skipped_checks, [])
        report.print_console()

    def test_runner_honors_enforcement_rules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = os.path.join(current_dir, "resources")
        runner = Runner()
        filter = RunnerFilter(framework=['circleci_pipelines'], use_enforcement_rules=True)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        filter.enforcement_rule_configs = {CheckType.CIRCLECI_PIPELINES: Severities[BcSeverities.OFF]}
        report = runner.run(
            root_folder=valid_dir_path,
            runner_filter=filter
        )
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.parsing_errors), 0)
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 0)
        report.print_console()

@pytest.mark.parametrize(
    "key, expected_key, supported_entities, start_line, end_line",
    [
        (
            'orbs.{orbs: @}.orbs.CKV_CIRCLECIPIPELINES_4[3:6]',
            "orbs",
            ('orbs.{orbs: @}',),
            3, 6
        ),
        (
            'jobs.*.steps[].jobs.*.steps[].CKV_CIRCLECIPIPELINES_7[48:49]',
            "jobs(test-echo).steps[1](checkout)",
            ('jobs.*.steps[]',),
            48, 49
        ),
        (
            'jobs.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}.jobs.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}.CKV_CIRCLECIPIPELINES_2[33:34]',
            'jobs(test-docker-versioned-img).docker.image[1](mongo:2.6.8)',
            ('jobs.*.docker[].{image: image, __startline__: __startline__, __endline__:__endline__}', ),
            33, 34
        )
    ]
)
def test_get_resource(file_path, key, supported_entities, expected_key, start_line, end_line, definition):
    runner = Runner()
    runner.definitions = definition

    new_key = runner.get_resource(file_path, key, supported_entities, start_line, end_line)

    assert new_key == expected_key


if __name__ == "__main__":
    unittest.main()



"""Module that checks if there is an updated version of a package available."""
from __future__ import annotations

import json
import os
import re
import sys
import time
from collections.abc import Generator, Callable
from datetime import datetime
from functools import wraps
from tempfile import gettempdir
from typing import Any

import platformdirs
import requests

CACHE_DIR_NAME = "checkov"
CACHE_FILENAME = "update_checker_cache.json"
LEGACY_CACHE_FILENAME = "update_checker_cache.pkl"
CACHE_EXPIRE_SECONDS = 3600
KEY_SEPARATOR = "||"
DATE_FORMAT = "%Y-%m-%dT%H:%M:%S"


def _get_cache_dir() -> str:
    """Return a per-user cache directory, creating it if needed."""
    cache_dir = platformdirs.user_cache_dir(CACHE_DIR_NAME)
    os.makedirs(cache_dir, exist_ok=True)
    return cache_dir


def _remove_legacy_cache() -> None:
    try:
        legacy_path = os.path.join(gettempdir(), LEGACY_CACHE_FILENAME)
        if os.path.exists(legacy_path):
            os.remove(legacy_path)
    except OSError:
        pass


def _serialize_result(result: UpdateResult | None) -> dict[str, Any] | None:
    if result is None:
        return None
    return {
        "package_name": result.package_name,
        "running_version": result.running_version,
        "available_version": result.available_version,
        "release_date": result.release_date.strftime(DATE_FORMAT) if result.release_date else None,
    }


def _deserialize_result(data: Any) -> UpdateResult | None:
    if not isinstance(data, dict):
        return None
    required = {"package_name", "running_version", "available_version", "release_date"}
    if not required.issubset(data.keys()):
        return None
    return UpdateResult(
        package=str(data["package_name"]),
        running=str(data["running_version"]),
        available=str(data["available_version"]),
        release_date=data.get("release_date"),
    )


def _serialize_cache(cache: dict[tuple[str, str], tuple[float, UpdateResult | None]]) -> dict[str, Any]:
    return {
        f"{pkg}{KEY_SEPARATOR}{ver}": [ts, _serialize_result(result)]
        for (pkg, ver), (ts, result) in cache.items()
    }


def _deserialize_cache(data: Any) -> dict[tuple[str, str], tuple[float, UpdateResult | None]]:
    cache: dict[tuple[str, str], tuple[float, UpdateResult | None]] = {}
    if not isinstance(data, dict):
        return cache
    for key, value in data.items():
        if not isinstance(key, str) or KEY_SEPARATOR not in key:
            continue
        parts = key.split(KEY_SEPARATOR, 1)
        if not isinstance(value, list) or len(value) != 2 or not isinstance(value[0], (int, float)):
            continue
        cache[(parts[0], parts[1])] = (float(value[0]), _deserialize_result(value[1]))
    return cache


def _write_json_atomic(path: str, data: Any) -> None:
    """Write JSON to *path* atomically via a temp file to avoid corruption."""
    tmp_path = path + ".tmp"
    with open(tmp_path, "w") as fp:
        json.dump(data, fp)
    os.replace(tmp_path, path)


def cache_results(
    function: Callable[[UpdateChecker, str, str], UpdateResult | None]
) -> Callable[[UpdateChecker, str, str], UpdateResult | None]:
    """Return decorated function that caches the results."""

    def _ensure_initialized() -> None:
        """Lazily initialize the cache directory and load the permacache on first use.

        """
        if _state["initialized"]:
            return
        _state["initialized"] = True
        try:
            _remove_legacy_cache()
            _state["filename"] = os.path.join(_get_cache_dir(), CACHE_FILENAME)
            update_from_permacache()
        except (NotImplementedError, OSError):
            _state["filename"] = None

    def save_to_permacache() -> None:
        """Merge in-memory cache into the on-disk permacache."""
        update_from_permacache()
        try:
            if _state["filename"] is None:
                return
            _write_json_atomic(_state["filename"], _serialize_cache(cache))
        except IOError:
            pass

    def update_from_permacache() -> None:
        """Load newer entries from the on-disk permacache into memory."""
        try:
            if _state["filename"] is None:
                return
            with open(_state["filename"], "r") as fp:
                permacache = _deserialize_cache(json.load(fp))
        except Exception:
            return
        for key, value in permacache.items():
            if key not in cache or value[0] > cache[key][0]:
                cache[key] = value

    cache: dict[tuple[str, str], tuple[float, UpdateResult | None]] = {}
    _state: dict[str, Any] = {"initialized": False, "filename": None}

    @wraps(function)
    def wrapped(obj: UpdateChecker, package_name: str, package_version: str, **extra_data: Any) -> UpdateResult | None:
        """Return cached results if available."""
        _ensure_initialized()
        now = time.time()
        key = (package_name, package_version)
        if not obj._bypass_cache and key in cache:
            cache_time, retval = cache[key]
            if now - cache_time < CACHE_EXPIRE_SECONDS:
                return retval
        retval = function(obj, package_name, package_version, **extra_data)
        cache[key] = now, retval
        if _state["filename"]:
            save_to_permacache()
        return retval

    return wrapped


def query_pypi(package: str, include_prereleases: bool) -> dict[str, Any]:
    """Return information about the current version of package."""
    try:
        response = requests.get(f"https://pypi.org/pypi/{package}/json", timeout=1)
    except requests.exceptions.RequestException:
        return {"success": False}
    if response.status_code != 200:
        return {"success": False}
    data = response.json()
    versions = list(data["releases"].keys())
    versions.sort(key=parse_version, reverse=True)

    version = versions[0]
    for tmp_version in versions:
        if include_prereleases or standard_release(tmp_version):
            version = tmp_version
            break

    upload_time = None
    for file_info in data["releases"][version]:
        if file_info["upload_time"]:
            upload_time = file_info["upload_time"]
            break

    return {"success": True, "data": {"upload_time": upload_time, "version": version}}


def standard_release(version: str) -> bool:
    return version.replace(".", "").isdigit()


class UpdateResult:
    """Contains the information for a package that has an update."""

    def __init__(self, package: str, running: str, available: str, release_date: str | None) -> None:
        self.available_version = available
        self.package_name = package
        self.running_version = running
        self.release_date: datetime | None = (
            datetime.strptime(release_date, DATE_FORMAT) if release_date else None
        )

    def __str__(self) -> str:
        """Return a printable UpdateResult string."""
        retval = f"Version {self.running_version} of {self.package_name} is outdated. Version {self.available_version} "
        if self.release_date:
            retval += f"was released {pretty_date(self.release_date)}."
        else:
            retval += "is available."
        return retval


class UpdateChecker:
    """A class to check for package updates."""

    def __init__(self, *, bypass_cache: bool = False) -> None:
        self._bypass_cache = bypass_cache

    @cache_results
    def check(self, package_name: str, package_version: str) -> UpdateResult | None:
        """Return a UpdateResult object if there is a newer version."""

        data = query_pypi(package_name, include_prereleases=not standard_release(package_version))

        if not data.get("success") or (parse_version(package_version) >= parse_version(data["data"]["version"])):
            return None

        return UpdateResult(
            package_name,
            running=package_version,
            available=data["data"]["version"],
            release_date=data["data"]["upload_time"],
        )


def pretty_date(the_datetime: datetime) -> str:
    """Attempt to return a human-readable time delta string."""
    # Source modified from
    # http://stackoverflow.com/a/5164027/176978
    diff = datetime.utcnow() - the_datetime
    if diff.days > 7 or diff.days < 0:
        return the_datetime.strftime("%A %B %d, %Y")
    elif diff.days == 1:
        return "1 day ago"
    elif diff.days > 1:
        return f"{diff.days} days ago"
    elif diff.seconds <= 1:
        return "just now"
    elif diff.seconds < 60:
        return f"{diff.seconds} seconds ago"
    elif diff.seconds < 120:
        return "1 minute ago"
    elif diff.seconds < 3600:
        return f"{int(round(diff.seconds / 60))} minutes ago"
    elif diff.seconds < 7200:
        return "1 hour ago"
    else:
        return f"{int(round(diff.seconds / 3600))} hours ago"


def update_check(package_name: str, package_version: str, bypass_cache: bool = False) -> None:
    """Convenience method that outputs to stderr if an update is available."""
    checker = UpdateChecker(bypass_cache=bypass_cache)
    result = checker.check(package_name, package_version)
    if result:
        print(result, file=sys.stderr)


# The following section of code is taken from setuptools pkg_resources.py (PSF
# license). Unfortunately importing pkg_resources to directly use the
# parse_version function results in some undesired side effects.

component_re = re.compile(r"(\d+ | [a-z]+ | \.| -)", re.VERBOSE)
replace = {"pre": "c", "preview": "c", "-": "final-", "rc": "c", "dev": "@"}.get


def _parse_version_parts(s: str) -> Generator[str, None, None]:
    for part in component_re.split(s):
        part = replace(part, part)
        if not part or part == ".":
            continue
        if part[:1] in "0123456789":
            yield part.zfill(8)  # pad for numeric comparison
        else:
            yield "*" + part

    yield "*final"  # ensure that alpha/beta/candidate are before final


def parse_version(s: str) -> tuple[str, ...]:
    """Convert a version string to a chronologically-sortable key.

    This is a rough cross between distutils' StrictVersion and LooseVersion;
    if you give it versions that would work with StrictVersion, then it behaves
    the same; otherwise it acts like a slightly-smarter LooseVersion. It is
    *possible* to create pathological version coding schemes that will fool
    this parser, but they should be very rare in practice.

    The returned value will be a tuple of strings.  Numeric portions of the
    version are padded to 8 digits so they will compare numerically, but
    without relying on how numbers compare relative to strings.  Dots are
    dropped, but dashes are retained.  Trailing zeros between alpha segments
    or dashes are suppressed, so that e.g. "2.4.0" is considered the same as
    "2.4". Alphanumeric parts are lower-cased.

    The algorithm assumes that strings like "-" and any alpha string that
    alphabetically follows "final"  represents a "patch level".  So, "2.4-1"
    is assumed to be a branch or patch of "2.4", and therefore "2.4.1" is
    considered newer than "2.4-1", which in turn is newer than "2.4".

    Strings like "a", "b", "c", "alpha", "beta", "candidate" and so on (that
    come before "final" alphabetically) are assumed to be pre-release versions,
    so that the version "2.4" is considered newer than "2.4a1".

    Finally, to handle miscellaneous cases, the strings "pre", "preview", and
    "rc" are treated as if they were "c", i.e. as though they were release
    candidates, and therefore are not as new as a version string that does not
    contain them, and "dev" is replaced with an '@' so that it sorts lower than
    than any other pre-release tag.

    """
    parts: list[str] = []
    for part in _parse_version_parts(s.lower()):
        if part.startswith("*"):
            if part < "*final":  # remove '-' before a prerelease tag
                while parts and parts[-1] == "*final-":
                    parts.pop()
            # remove trailing zeros from each series of numeric parts
            while parts and parts[-1] == "00000000":
                parts.pop()
        parts.append(part)
    return tuple(parts)



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.SynapseWorkspaceEnablesManagedVirtualNetworks import check
from checkov.common.models.enums import CheckResult


class TestSynapseWorkspaceEnablesManagedVirtualNetworks(unittest.TestCase):

    def test_failure_1(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_synapse_workspace" "example" {
              name                                 = "example"
              resource_group_name                  = azurerm_resource_group.example.name
              location                             = azurerm_resource_group.example.location
              storage_data_lake_gen2_filesystem_id = azurerm_storage_data_lake_gen2_filesystem.example.id
              sql_administrator_login              = "sqladminuser"
              sql_administrator_login_password     = "H@Sh1CoR3!"  # checkov:skip=CKV_SECRET_80 test secret
              managed_virtual_network_enabled      = false
              aad_admin {
                login     = "AzureAD Admin"
                object_id = "00000000-0000-0000-0000-000000000000"
                tenant_id = "00000000-0000-0000-0000-000000000000"
              }
            
              tags = {
                Env = "production"
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_synapse_workspace']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_2(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_synapse_workspace" "example" {
              name                                 = "example"
              resource_group_name                  = azurerm_resource_group.example.name
              location                             = azurerm_resource_group.example.location
              storage_data_lake_gen2_filesystem_id = azurerm_storage_data_lake_gen2_filesystem.example.id
              sql_administrator_login              = "sqladminuser"
              sql_administrator_login_password     = "H@Sh1CoR3!"  # checkov:skip=CKV_SECRET_80 test secret
              aad_admin {
                login     = "AzureAD Admin"
                object_id = "00000000-0000-0000-0000-000000000000"
                tenant_id = "00000000-0000-0000-0000-000000000000"
              }

              tags = {
                Env = "production"
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_synapse_workspace']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_synapse_workspace" "example" {
              name                                 = "example"
              resource_group_name                  = azurerm_resource_group.example.name
              location                             = azurerm_resource_group.example.location
              storage_data_lake_gen2_filesystem_id = azurerm_storage_data_lake_gen2_filesystem.example.id
              sql_administrator_login              = "sqladminuser"
              sql_administrator_login_password     = "H@Sh1CoR3!"  # checkov:skip=CKV_SECRET_80 test secret
              managed_virtual_network_enabled      = true                
              aad_admin {
                login     = "AzureAD Admin"
                object_id = "00000000-0000-0000-0000-000000000000"
                tenant_id = "00000000-0000-0000-0000-000000000000"
              }

              tags = {
                Env = "production"
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_synapse_workspace']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from checkov.common.images.image_referencer import Image
from checkov.dockerfile.image_referencer.manager import DockerfileImageReferencerManager


def test_extract_images_from_resources():
    # given
    definitions = {
        "/Dockerfile": {
            "FROM": [
                {
                    "instruction": "FROM",
                    "startline": 0,
                    "endline": 0,
                    "content": "FROM php:7.1-apache\n",
                    "value": "php:7.1-apache",
                }
            ],
            "RUN": [
                {
                    "instruction": "RUN",
                    "startline": 2,
                    "endline": 2,
                    "content": "RUN apk --no-cache add nginx\n",
                    "value": "apk --no-cache add nginx",
                },
            ],
        },
    }

    # when
    images = DockerfileImageReferencerManager(definitions=definitions).extract_images_from_resources()

    # then
    assert images == [
        Image(
            file_path="/Dockerfile",
            name="php:7.1-apache",
            start_line=1,
            end_line=1,
            related_resource_id="/Dockerfile:/Dockerfile.FROM",
        ),
    ]



from aws_cdk import core
from aws_cdk import aws_apigateway as apigateway
from aws_cdk import aws_sam as sam
class MyApiGatewayStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create an API Gateway stage with cache cluster enabled
        api = apigateway.RestApi(
            self,
            "MyApi",
            rest_api_name="MyApiName",
        )

        stage = apigateway.Stage(
            self,
            "MyApiStage",
            stage_name="prod",  # Replace with your desired stage name
            deployment=api.latest_deployment,
            cache_cluster_enabled=True,  # Enable cache cluster
            cache_cluster_size="0.5",  # Specify the cache cluster size
        )

class MySAMApiStack2(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create a Serverless API with cache cluster enabled
        sam_api = sam.CfnApi(
            self,
            "MySAMApi",
            cacheClusterEnabled=True,  # Enable cache cluster
            cacheClusterSize="0.5",  # Specify the cache cluster size
            stage_name="prod",  # Specify the stage name
            definition_body={
                "openapi": "3.0.1",
                "info": {
                    "title": "MyAPI",
                },
                "paths": {
                    "/example": {
                        "get": {
                            "responses": {
                                "200": {
                                    "description": "A sample response",
                                },
                            },
                        },
                    },
                },
            },
        )



import unittest

from detect_secrets.util.code_snippet import CodeSnippet

from checkov.secrets.parsers.json.multiline_parser import json_multiline_parser


class TestMultilineParserJson(unittest.TestCase):
	def setUp(self) -> None:
		self.json_parser = json_multiline_parser

	def test_is_object_start(self):
		examples = [
			(True,  '   {'),
			(True,  '   {	\n'),
			(True,  '		{'),
			(True,  '{'),
			(True,  '"key": {'),
			(True,  '}, {'),
			(False, '"key": {}'),
			(False, '"key": { }'),
			(False, '"key": {	},	'),
			(False, '{ "key": "value" }'),
			(False, '"key": { "key2": "value2", "key3": "value3" }, '),
			(False, '"key": "value" }'),
			(False, '	}, { "key":  '),  # is not supported
			(False, '}'),
			(False, '		}'),
			(False, '"key": 1, '),
			(False, '[1,2], ['),
			(False, '[1,2], '),
		]

		for ans, line in examples:
			assert ans == self.json_parser.is_object_start(line)

	def test_is_object_end(self):
		examples = [
			(True,  '}'),
			(True,  '		}'),
			(True,  '   }'),
			(True,  '	},'),
			(True,  '}]}, '),
			(True,  '"key": "value" }'),
			(True,  '"key": "value" } , '),
			(True,  '}, {'),
			(False,  '"key": {}'),
			(False,  '"key": { }'),
			(False,  '"key": {	},	'),
			(True,  '	}, { "key": "value",'),
			(False, '"key": { "key2": "value2", "key3": "value3" }, '),
			(False, '"key": {'),
			(False, '{'),
			(False, '		{'),
			(False, '"key": 1, '),
			(False, '[1,2], '),
		]

		for ans, line in examples:
			assert ans == self.json_parser.is_object_end(line)

	def test_lines_in_same_object1(self):
		# Please notice that additional logic required to determine for non-sequential lines,
		# see EntropyKeywordCombinator.analyze_multiline() function for example.
		# In the context of the tested function, 2 lines are treated as sequential.
		raw_context_example = CodeSnippet(
			snippet=[
				'  conf_list_of_dicts": [',
				'    {',
				'      "name": "TEST_PASSWORD_1",',
				'      "value": "some secret",',
				'      "desc": "still same obj",',
				'      "type": 1',
				'    }',
				'  ],',
				'  "conf_dict": {',
				'    "name": "TEST_PASSWORD_1",',
				'    "value": "some secret",'
			],
			start_line=5,
			target_index=5
		)
		examples = [
			# index of rows in the context compared to the target row, which is:
			# '      "type": 1'
			(True, 1, raw_context_example),
			(True, 2, raw_context_example),
			(True, 3, raw_context_example),
			(True, 4, raw_context_example),
			(True, 6, raw_context_example),
		]

		for res, i, raw_context in examples:
			assert res == self.json_parser.consecutive_lines_in_same_object(
				raw_context=raw_context, other_line_idx=i)

	def test_lines_in_same_object2(self):

		raw_context_example = CodeSnippet(
			snippet=[
				'  , {',
				'      "value": "some secret",',
				'    }],',
				'  "conf_dict": {',
				'    "name": {} ',
				'    "value": { "key2": "value2", "key3": "value3" }",'
			],
			start_line=2,
			target_index=1
		)

		examples = [
			# index of rows in the context compared to the target row, which is:
			# '      "value": "some secret"'
			(False, 0, raw_context_example),
			(True, 2, raw_context_example),
			(True, 3, raw_context_example),
		]

		for res, i, raw_context in examples:
			assert res == self.json_parser.consecutive_lines_in_same_object(
				raw_context=raw_context, other_line_idx=i)

	def test_lines_in_same_object3(self):
		raw_context_example = CodeSnippet(
			snippet=[
				'  , {',
				'      "value": "some secret",',
				'    }],',
				'  "conf_dict": {',
				'    "name": {} ',
				'    "value": { ',
				'  	 	"key2": "value2", ',
				'    	"value": { "key2": "value2", "key3": "value3" }, ',
			],
			start_line=2,
			target_index=5
		)

		examples = [
			# index of rows in the context compared to the target row, which is:
			# '    "name": {} '
			(True, 4, raw_context_example),
			(False, 6, raw_context_example),
		]

		for res, i, raw_context in examples:
			assert res == self.json_parser.consecutive_lines_in_same_object(
				raw_context=raw_context, other_line_idx=i)



from aws_cdk import core
from aws_cdk import aws_codebuild as codebuild

class MyCodeBuildProjectStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define a CodeBuild project with S3 artifacts and encryption disabled
        my_project = codebuild.Project(
            self, 'MyCodeBuildProject',
            project_name='MyProject',
            source=codebuild.Source.git_hub(owner='owner', repo='repo'),
            artifacts=codebuild.Artifacts(
                type=codebuild.ArtifactsType.S3,
                encryption_disabled=False
            ),
            environment=codebuild.BuildEnvironment(build_image=codebuild.LinuxBuildImage.STANDARD_5_0),
        )

app = core.App()
MyCodeBuildProjectStack(app, "MyCodeBuildProjectStack")
app.synth()

class MyCodeBuildProjectStack2(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define a CodeBuild project with S3 artifacts and encryption disabled
        my_project = codebuild.Project(
            self, 'MyCodeBuildProject',
            project_name='MyProject',
            source=codebuild.Source.git_hub(owner='owner', repo='repo'),
            artifacts=codebuild.Artifacts(
                encryption_disabled=True
            ),
            environment=codebuild.BuildEnvironment(build_image=codebuild.LinuxBuildImage.STANDARD_5_0),
        )

app = core.App()
MyCodeBuildProjectStack2(app, "MyCodeBuildProjectStack2")
app.synth()


from __future__ import annotations

import json
import logging
import os
import platform
import threading
from pathlib import Path
from typing import Any, cast, Optional, TextIO, Type

import hcl2

from checkov.common.util.env_vars_config import env_vars_config
from checkov.common.util.stopit import ThreadingTimeout, SignalTimeout
from checkov.common.util.stopit.utils import BaseTimeout
from checkov.terraform import validate_malformed_definitions, clean_bad_definitions
from checkov.terraform.modules.module_utils import _Hcl2Payload


def load_or_die_quietly(
    file: str | Path | os.DirEntry[str], parsing_errors: dict[str, Exception], clean_definitions: bool = True
) -> Optional[_Hcl2Payload]:
    """
    Load JSON or HCL, depending on filename.
    :return: None if the file can't be loaded
    """
    file_path = os.fspath(file)
    file_name = os.path.basename(file_path)

    if file_name.endswith(".tfvars"):
        clean_definitions = False

    try:
        logging.debug(f"Parsing {file_path}")

        with open(file_path, "r", encoding="utf-8-sig") as f:
            if file_name.endswith(".json"):
                return cast("_Hcl2Payload", json.load(f))
            else:
                raw_data = __parse_with_timeout(f)
                non_malformed_definitions = validate_malformed_definitions(raw_data)
                if clean_definitions:
                    return clean_bad_definitions(non_malformed_definitions)
                else:
                    return non_malformed_definitions
    except Exception as e:
        logging.debug(f"failed while parsing file {file_path}", exc_info=True)
        parsing_errors[file_path] = e
        return None


# if we are not running in a thread, run the hcl2.load function with a timeout, to prevent from getting stuck in parsing.
def __parse_with_timeout(f: TextIO) -> dict[str, list[dict[str, Any]]]:
    # setting up timeout class
    timeout_class: Optional[Type[BaseTimeout]] = None
    if platform.system() == "Windows":
        timeout_class = ThreadingTimeout
    elif threading.current_thread() is threading.main_thread():
        timeout_class = SignalTimeout

    # if we're not running on the main thread, don't use timeout
    parsing_timeout = env_vars_config.HCL_PARSE_TIMEOUT_SEC or 0
    if not timeout_class or not parsing_timeout:
        return hcl2.load(f)

    with timeout_class(parsing_timeout) as to_ctx_mgr:
        raw_data = hcl2.load(f)
    if to_ctx_mgr.state == to_ctx_mgr.TIMED_OUT:
        logging.debug(f"reached timeout when parsing file {f} using hcl2")
        raise Exception(f"file took more than {parsing_timeout} seconds to parse")
    return raw_data



from aws_cdk import aws_apigateway as apigateway

cfn_stage = apigateway.CfnStage(self, "MyCfnStage",
    rest_api_id="restApiId",

    # the properties below are optional

    cache_cluster_enabled=False,
    cache_cluster_size="cacheClusterSize",
    canary_setting=apigateway.CfnStage.CanarySettingProperty(
        deployment_id="deploymentId",
        percent_traffic=123,
        stage_variable_overrides={
            "stage_variable_overrides_key": "stageVariableOverrides"
        },
        use_stage_cache=False
    ),
    client_certificate_id="clientCertificateId",
    deployment_id="deploymentId",
    description="description",
    documentation_version="documentationVersion",
    method_settings=[apigateway.CfnStage.MethodSettingProperty(
        cache_data_encrypted=False,
        cache_ttl_in_seconds=123,
        caching_enabled=False,
        data_trace_enabled=False,
        http_method="httpMethod",
        logging_level="loggingLevel",
        metrics_enabled=False,
        resource_path="resourcePath",
        throttling_burst_limit=123,
        throttling_rate_limit=123
    )],
    stage_name="stageName",
    tags=[CfnTag(
        key="key",
        value="value"
    )],
    tracing_enabled=False,
    variables={
        "variables_key": "variables"
    }
)

from aws_cdk import core
from aws_cdk import aws_serverless as serverless

class ServerlessApiWithAccessLogStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create a Serverless API
        serverless.Api(
            self, "MyApi",
            default_stage={
                "stage_name": "prod",
                "access_log_setting": serverless.AccessLogSetting(
                    format=serverless.AccessLogFormat.json_with_standard_fields()
                )
            }
        )

app = core.App()
ServerlessApiWithAccessLogStack(app, "ServerlessApiWithAccessLogStack")
app.synth()



from __future__ import annotations

import itertools
import json
import os
from pathlib import Path

import yaml
from abc import abstractmethod
from typing import List, Optional, Any
from unittest import TestCase

from checkov.cloudformation.runner import Runner
from checkov.common.checks_infra.checks_parser import GraphCheckParser
from checkov.common.checks_infra.registry import Registry
from checkov.common.graph.graph_manager import GraphManager
from checkov.common.output.record import Record
from checkov.runner_filter import RunnerFilter


class TestYamlPoliciesBase(TestCase):
    def __init__(self, graph_manager: GraphManager, real_graph_checks_path: str,
                 test_checks_path: Optional[str], check_type: str, test_file_path: str,
                 args):
        super().__init__(args)
        self.check_type = check_type
        self.real_graph_checks_path = real_graph_checks_path
        self.checks_dir = test_checks_path
        self.test_file_path = test_file_path
        self.graph_manager = graph_manager

    def go(self, dir_name, check_name=None, local_graph_class=None):
        dir_path = os.path.join(os.path.dirname(os.path.realpath(self.test_file_path)),
                                f"resources/{dir_name}")
        assert os.path.exists(dir_path)
        if self.checks_dir:
            assert os.path.exists(self.checks_dir)
        found = False
        for root, d_names, f_names in itertools.chain(os.walk(self.real_graph_checks_path), os.walk(self.checks_dir)):
            for f_name in f_names:
                check_name = dir_name if check_name is None else check_name
                if f_name == f"{check_name}.yaml":
                    found = True
                    policy = load_yaml_data(f_name, root)
                    assert policy is not None
                    expected = load_yaml_data("expected.yaml", dir_path)
                    assert expected is not None
                    report = self.get_policy_results(dir_path, policy, local_graph_class)
                    expected = load_yaml_data("expected.yaml", dir_path)

                    expected_to_fail = expected.get('fail', [])
                    expected_to_pass = expected.get('pass', [])
                    expected_to_skip = expected.get('skip', [])
                    expected_evaluated_keys = expected.get('evaluated_keys', [])
                    self.assert_entities(expected_to_pass, report.passed_checks, True)
                    self.assert_entities(expected_to_fail, report.failed_checks, False)
                    self.assert_entities(expected_to_skip, report.skipped_checks, True)
                    self.assert_evaluated_keys(expected_evaluated_keys, report.passed_checks + report.failed_checks)

        assert found

    def assert_entities(self, expected_entities: List[str], results: List[Record], assertion: bool):
        self.assertEqual(len(expected_entities), len(results),
                         f"mismatch in number of results in {'passed' if assertion else 'failed'}, "
                         f"expected: {len(expected_entities)}, got: {len(results)}")
        for expected_entity in expected_entities:
            found = False
            for check_result in results:
                entity_id = check_result.resource
                if entity_id == expected_entity:
                    found = True
                    break
            self.assertTrue(found, f"expected to find entity {expected_entity}, {'passed' if assertion else 'failed'}")

    def get_policy_results(self, root_folder, policy, local_graph_class=None):
        check_id = policy['metadata']['id']
        local_graph, _ = self.graph_manager.build_graph_from_source_directory(
            source_dir=root_folder,
            local_graph_class=local_graph_class,
        )
        graph = self.graph_manager.save_graph(local_graph)
        registry = self.get_checks_registry()
        checks_results = registry.run_checks(graph, RunnerFilter(checks=[check_id]), None)
        return self.create_report_from_graph_checks_results(checks_results, policy['metadata'])

    def get_checks_registry(self):
        registry = Registry(parser=GraphCheckParser(), checks_dir=self.real_graph_checks_path)
        registry.load_checks()
        if self.checks_dir:
            registry.load_external_checks(self.checks_dir)
        return registry

    @abstractmethod
    def create_report_from_graph_checks_results(self, checks_results, check):
        pass

    @abstractmethod
    def assert_evaluated_keys(self, checks_results, check):
        pass


def load_yaml_data(source_file_name: str | Path, dir_path: str | Path) -> Any:
    expected_path = os.path.join(dir_path, source_file_name)
    if not os.path.exists(expected_path):
        return None

    with open(expected_path, "r") as f:
        expected_data = yaml.safe_load(f)

    return json.loads(json.dumps(expected_data))


def get_expected_results_by_file_name(test_dir: str | Path) -> (list[str], list[str]):
    if not os.path.exists(test_dir):
        return None
    expected_fail = []
    expected_pass = []
    for root, d_names, f_names in os.walk(test_dir):
        for file in f_names:
            if file.startswith('fail'):
                expected_fail.append(file)
            elif file.startswith('pass'):
                expected_pass.append(file)
            else:
                raise NameError('yaml test files should start with either pass / fail')

    return (expected_fail, expected_pass)


def get_policy_results(root_folder, policy):
    check_id = policy['metadata']['id']
    graph_runner = Runner()
    report = graph_runner.run(root_folder, runner_filter=RunnerFilter(checks=[check_id]))
    return report


from typing import List

from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories


class GoogleVertexAINotebookShieldedVM(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure Vertex AI Notebook instances are launched with Shielded VM enabled"
        id = "CKV_GCP_126"
        supported_resources = ['google_notebooks_instance']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'shielded_instance_config/[0]/enable_vtpm'

    def get_forbidden_values(self) -> List[bool]:
        return [False]


check = GoogleVertexAINotebookShieldedVM()



"""
Utility to strip common build log prefixes from file content.

When scanning build logs for secrets, each line may be prefixed with timestamps
and log metadata (e.g., "2026-01-07 09:41:37.553 | DEBUG | crypto      | ").
These prefixes break multiline regex matching for secrets (like private keys)
because the detectors expect clean content between markers.

This module provides functions to detect and strip such prefixes, enabling
secret detection across all detectors when scanning build log files.
"""
from __future__ import annotations

import re
from typing import Optional

# Common build log prefix patterns:
# - Timestamps: 2026-01-07 09:41:37.553, 2026-01-07T09:41:37.553Z
# - Log levels: INFO, DEBUG, WARN, ERROR, TRACE, FATAL
# - Separators: |, -, [], etc.
# - Module names: crypto, main, etc.
# Examples:
#   "2026-01-07 09:41:37.553 | DEBUG | crypto      | actual content"
#   "[2026-01-07 09:41:37] [INFO] actual content"
#   "2026-01-07T09:41:37.553Z INFO  actual content"
#   "09:41:37.553 DEBUG actual content"

# Pattern for pipe-separated log prefixes (most common in CI/CD logs)
# e.g., "2026-01-07 09:41:37.553 | DEBUG | crypto      | "
_PIPE_LOG_PREFIX = re.compile(
    r'^'
    r'(?:'
    # Date-time with various separators
    r'\d{4}[-/]\d{2}[-/]\d{2}[T ]\d{2}:\d{2}:\d{2}(?:[.,]\d+)?(?:Z|[+-]\d{2}:?\d{2})?'
    r'|'
    # Time-only prefix
    r'\d{2}:\d{2}:\d{2}(?:[.,]\d+)?'
    r')'
    # Pipe-separated fields (log level, module, etc.)
    r'(?:\s*\|\s*\w[\w./\- ]*)*'
    # Final pipe separator before actual content
    r'\s*\|\s*'
)

# Pattern for space-separated log prefixes
# e.g., "2026-01-07 09:41:37.553 DEBUG crypto - "
_SPACE_LOG_PREFIX = re.compile(
    r'^'
    r'(?:'
    r'\d{4}[-/]\d{2}[-/]\d{2}[T ]\d{2}:\d{2}:\d{2}(?:[.,]\d+)?(?:Z|[+-]\d{2}:?\d{2})?'
    r'|'
    r'\d{2}:\d{2}:\d{2}(?:[.,]\d+)?'
    r')'
    r'\s+'
    # Log level (required for space-separated to avoid false positives)
    r'(?:TRACE|DEBUG|INFO|WARN(?:ING)?|ERROR|FATAL|SEVERE|FINE|FINER|FINEST)\s+'
    # Optional module/category and separator
    r'(?:[\w./-]+\s+(?:[-:]\s+)?)?'
)

# Pattern for bracket-style log prefixes
# e.g., "[2026-01-07 09:41:37] [INFO] [crypto] "
_BRACKET_LOG_PREFIX = re.compile(
    r'^'
    r'(?:\[[\d\-/:T., +Z]+\]\s*)'
    r'(?:\[(?:TRACE|DEBUG|INFO|WARN(?:ING)?|ERROR|FATAL|SEVERE)\]\s*)?'
    r'(?:\[[\w./-]+\]\s*)*'
)

_LOG_PREFIX_PATTERNS = [_PIPE_LOG_PREFIX, _SPACE_LOG_PREFIX, _BRACKET_LOG_PREFIX]


def strip_log_prefix(line: str) -> str:
    """Strip common build log prefixes from a single line.

    Removes timestamp, log level, and module prefixes commonly found in build logs.
    Returns the line content after the prefix.
    """
    for pattern in _LOG_PREFIX_PATTERNS:
        stripped = pattern.sub('', line)
        if stripped != line:
            return stripped
    return line


def has_log_prefixes(content: str) -> bool:
    """Check if file content appears to be a build log with line prefixes.

    Samples non-empty lines and checks if a significant portion have log prefixes.
    Returns True if the file appears to be a log file with prefixes.
    """
    lines = content.split('\n')
    prefix_count = 0
    checked = 0
    sample_size = min(len(lines), 30)  # Check up to 30 non-empty lines

    for line in lines:
        if not line.strip():
            continue
        stripped = strip_log_prefix(line)
        if stripped != line:
            prefix_count += 1
        checked += 1
        if checked >= sample_size:
            break

    # Require at least 2 lines with prefixes and at least 30% of sampled lines
    return checked > 0 and prefix_count >= 2 and (prefix_count / checked) >= 0.3


def strip_log_prefixes_from_content(content: str) -> str:
    """Strip log prefixes from all lines in file content.

    Returns the content with log prefixes removed from each line.
    """
    lines = content.split('\n')
    stripped_lines = [strip_log_prefix(line) for line in lines]
    return '\n'.join(stripped_lines)


def create_stripped_content(file_path: str) -> Optional[str]:
    """Read a file and return its content with log prefixes stripped.

    Returns a string of the stripped content, or None if the file doesn't appear to have log prefixes.
    """
    try:
        # To avoid reading large files entirely into memory just for a check,
        # we first read a sample of the file to check for log prefixes.
        with open(file_path, 'r') as f:
            sample = f.read(8000)  # Read first 8KB, should be enough for several lines
    except (OSError, UnicodeDecodeError):
        return None

    if not has_log_prefixes(sample):
        return None

    # Prefixes were found in the sample, so now we process the whole file.
    # The runner that calls this already limits the file size, so we don't
    # expect to be reading huge files here.
    try:
        with open(file_path, 'r') as f:
            content = f.read()
        return strip_log_prefixes_from_content(content)
    except (OSError, UnicodeDecodeError):
        return None



import unittest
import hcl2

from checkov.terraform.checks.resource.aws.RDSEnhancedMonitorEnabled import check
from checkov.common.models.enums import CheckResult


class TestRDSEnhancedMonitorEnabled(unittest.TestCase):

    def test_failure1(self):
        hcl_res = hcl2.loads("""
resource "aws_db_instance" "default" {
  allocated_storage    = 10
  engine               = "mysql"
  engine_version       = "5.7"
  instance_class       = "db.t3.micro"
  name                 = "mydb"
  username             = "foo"
  password             = "foobarbaz"
  parameter_group_name = "default.mysql5.7"
  skip_final_snapshot  = true
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_db_instance']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure2(self):
        hcl_res = hcl2.loads("""
resource "aws_db_instance" "default" {
  allocated_storage    = 10
  engine               = "mysql"
  engine_version       = "5.7"
  instance_class       = "db.t3.micro"
  name                 = "mydb"
  username             = "foo"
  password             = "foobarbaz"
  parameter_group_name = "default.mysql5.7"
  skip_final_snapshot  = true
  monitoring_interval  = 0
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_db_instance']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure3(self):
        hcl_res = hcl2.loads("""
resource "aws_db_instance" "default" {
  allocated_storage    = 10
  engine               = "mysql"
  engine_version       = "5.7"
  instance_class       = "db.t3.micro"
  name                 = "mydb"
  username             = "foo"
  password             = "foobarbaz"
  parameter_group_name = "default.mysql5.7"
  skip_final_snapshot  = true
  monitoring_interval  = "5"
}
        """)
        resource_conf = hcl_res['resource'][0]['aws_db_instance']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success1(self):
        hcl_res = hcl2.loads("""
resource "aws_db_instance" "default" {
  allocated_storage    = 10
  engine               = "mysql"
  engine_version       = "5.7"
  instance_class       = "db.t3.micro"
  name                 = "mydb"
  username             = "foo"
  password             = "foobarbaz"
  parameter_group_name = "default.mysql5.7"
  skip_final_snapshot  = true
  monitoring_interval  = 5
  }
           """)
        resource_conf = hcl_res['resource'][0]['aws_db_instance']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success2(self):
        hcl_res = hcl2.loads("""
resource "aws_db_instance" "default" {
  allocated_storage    = 10
  engine               = "mysql"
  engine_version       = "5.7"
  instance_class       = "db.t3.micro"
  name                 = "mydb"
  username             = "foo"
  password             = "foobarbaz"
  parameter_group_name = "default.mysql5.7"
  skip_final_snapshot  = true
  monitoring_interval  = 15
  }
           """)
        resource_conf = hcl_res['resource'][0]['aws_db_instance']['default']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import pytest

from checkov.common.output.record import Record
from checkov.policies_3d.syntax.iac_syntax import ViolationIdEquals


@pytest.fixture
def record() -> Record:
    _check_id = 'CHECK_ID'
    return Record(
        bc_check_id=_check_id,
        check_id=_check_id,
        check_name='mock',
        check_result={'result': 'failed'},
        code_block=[],
        file_path='',
        file_line_range=[],
        resource='',
        check_class='',
        file_abs_path='',
        evaluations={}
    )

def test_violation_id_equals_predicate_true(record: Record):
    # Arrange
    check_id = 'CHECK_ID'
    predicate = ViolationIdEquals(record, check_id)

    # Act
    predicate()

    # Assert
    assert predicate.is_true


def test_violation_id_equals_predicate_false(record: Record):
    # Arrange
    check_id = 'NOT_A_CHECK_ID'
    predicate = ViolationIdEquals(record, check_id)

    # Act
    predicate()

    # Assert
    assert not predicate.is_true


def test_violation_id_equals_predicate_false_equality(record: Record):
    # Arrange
    check_id_1 = 'CHECK_ID_1'
    check_id_2 = 'CHECK_ID_2'
    p1 = ViolationIdEquals(record, check_id_1)
    p2 = ViolationIdEquals(record, check_id_2)

    # Assert
    assert p1 != p2

def test_violation_id_equals_predicate_true_equality(record: Record):
    # Arrange
    check_id = 'CHECK_ID'
    p1 = ViolationIdEquals(record, check_id)
    p2 = ViolationIdEquals(record, check_id)

    # Assert
    assert p1 == p2




from operator import itemgetter
from pathlib import Path

from checkov.common.util.dockerfile import is_dockerfile
from checkov.dockerfile.graph_builder.graph_components.resource_types import ResourceType
from checkov.dockerfile.utils import get_files_definitions, build_definitions_context

VALID_DOCKERFILE_NAMES = [
    "Dockerfile",
    "dockerfile",
    "Dockerfile.prod",
    "Dockerfile.Product1",
    "dev.Dockerfile",
    "team1.product.dockerfile",
]
INVALID_DOCKERFILE_NAMES = [
    "package.json",
    "dockerfil",
    "dockerfilee",
    ".dockerfile",
    "ddockerfile",
    "ockerfile",
    "docker-file",
    "dockerfile1",
    "Dockerfile.env.dockerignore",
    "Dockerfile.env.Dockerignore",
    "dockerfile.dockerignore",
]


def test_is_dockerfile():
    assert all(is_dockerfile(curr_name) for curr_name in VALID_DOCKERFILE_NAMES)
    assert all(not is_dockerfile(curr_name) for curr_name in INVALID_DOCKERFILE_NAMES)


def test_build_definitions_context():
    # given
    file_path = Path(__file__).parent / "resources/expose_port/skip/Dockerfile"
    definitions, definitions_raw = get_files_definitions(files=[str(file_path)])

    # when
    context = build_definitions_context(definitions=definitions, definitions_raw=definitions_raw)

    assert len(context) == 1

    definition_context = next(iter(context.values()))
    assert len(definition_context) == 4

    run_instructions = definition_context[ResourceType.RUN]
    assert len(run_instructions) == 1
    run_instruction = run_instructions[0]
    assert run_instruction["start_line"] == 4
    assert run_instruction["end_line"] == 4
    assert run_instruction["code_lines"] == [(4, "RUN apk --no-cache add nginx\n")]

    for skip in run_instruction["skipped_checks"]:
        skip.pop("bc_id", None)  # depending on the test order they are set or not

    assert sorted(run_instruction["skipped_checks"], key=itemgetter("id")) == sorted(
        [
            {"id": "CKV_DOCKER_1", "line_number": 5, "suppress_comment": " required"},
            {"id": "CKV_DOCKER_5", "line_number": 0, "suppress_comment": " no need to skip"},
            {"id": "CKV2_DOCKER_7", "line_number": 1, "suppress_comment": " no need to skip graph check"},
        ],
        key=itemgetter("id"),
    )



import unittest
import configargparse

from checkov.common.util.ext_argument_parser import ExtArgumentParser


class TestConfigFile(unittest.TestCase):
    def test_pass(self):
        argv = ['--ca-certificate', '----- BEGIN CERTIFICATE ----- <KEY> ----- END CERTIFICATE -----',
                '--compact', '--directory', 'test-dir', '--docker-image', 'sample-image', '--dockerfile-path',
                'Dockerfile', '--download-external-modules', 'True', '--evaluate-variables', 'False',
                '--external-checks-dir', 'sample-dir', '--external-checks-git', 'sample-github-url', '--file',
                'sample.tf', '--framework', 'all', '--output', 'cli', '--quiet', '--repo-id',
                'bridgecrew/sample-repo', '--skip-check', 'CKV_DOCKER_3,CKV_DOCKER_2',
                '--skip-framework', 'dockerfile', '--soft-fail', '--branch', 'master',
                '--check', 'CKV_DOCKER_1']
        argv_parser = ExtArgumentParser(config_file_parser_class=configargparse.YAMLConfigFileParser)
        config_parser = ExtArgumentParser(
            config_file_parser_class=configargparse.YAMLConfigFileParser,
            default_config_files=['example_TestConfigFile/config.yml'],
        )

        argv_parser.add_parser_args()
        config_parser.add_parser_args()

        config_from_argv = argv_parser.parse_args(argv)
        config_from_file = config_parser.parse_args([])
        self.assertEqual(config_from_argv, config_from_file)


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.provider.openstack.credentials import check
from checkov.common.models.enums import CheckResult


class TestCredentials(unittest.TestCase):
    def test_success_empty(self):
        hcl_res = hcl2.loads(
            """
            provider "openstack" {}
            """
        )
        provider_conf = hcl_res["provider"][0]["openstack"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_region(self):
        hcl_res = hcl2.loads(
            """
            provider "openstack" {
                auth_url = "http://myauthurl:5000/v2.0"
                region   = "RegionOne"
            }
            """
        )
        provider_conf = hcl_res["provider"][0]["openstack"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure_password(self):
        hcl_res = hcl2.loads(
            """
            provider "openstack" {
                auth_url = "http://myauthurl:5000/v2.0"
                region   = "RegionOne"
                password = "Ahngak0fuokeexee5Quiu0oohayeiXie"
            }
            """
        )
        provider_conf = hcl_res["provider"][0]["openstack"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_token(self):
        hcl_res = hcl2.loads(
            """
            provider "openstack" {
                auth_url = "http://myauthurl:5000/v2.0"
                region   = "RegionOne"
                token    = "ifahghau4nun7eirahJ5baa8cichex7l"
            }
            """
        )
        provider_conf = hcl_res["provider"][0]["openstack"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_app_secret(self):
        hcl_res = hcl2.loads(
            """
            provider "openstack" {
                auth_url = "http://myauthurl:5000/v2.0"
                region   = "RegionOne"
                application_credential_secret = "mie8siw5ooTaed0AeQuepeiGhah9xaif"
            }
            """
        )
        provider_conf = hcl_res["provider"][0]["openstack"]
        scan_result = check.scan_provider_conf(conf=provider_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


if __name__ == "__main__":
    unittest.main()
